<?php
/* --------------------------------------------------------------
   paypal_config.php 2.5_2014-08-11_1722 mabr
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2013 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

require_once 'includes/application_top.php';
define('PAGE_URL', HTTP_SERVER.DIR_WS_ADMIN.basename(__FILE__));

if(isset($_SESSION['coo_page_token']))
{
	$t_page_token = $_SESSION['coo_page_token']->generate_token();
}
else
{
	$t_page_token = '';
}

$messages_ns = 'messages_'.basename(__FILE__);
if(!isset($_SESSION[$messages_ns])) {
	$_SESSION[$messages_ns] = array();
}

$paypal = new GMPayPal();
$config = $paypal->getConfigArray();

if(isset($_REQUEST['check_credentials'])) {
	$credentials_valid = $paypal->checkAPIAccess();
	if($credentials_valid === true || $credentials_valid === false) {
		echo '<p class="status '. ($credentials_valid ? 'status_ok' : 'status_fail') .'">';
		echo $credentials_valid ? $paypal->get_text('PPTXT_CONFIG_CREDENTIALS_VALID') : $paypal->get_text('PPTXT_CONFIG_CREDENTIALS_INVALID');
		echo $config['sandbox_mode'] == 1 ? '<br><strong>'.$paypal->get_text('PPTXT_CONFIG_SANDBOX_NOTICE').'</strong>' : '';
		echo '</p>';
	}
	else {
		echo '<p class="status status_fail">';
		echo $paypal->get_text('PPTXT_CONFIG_CREDENTIALS_ERROR').': '.$credentials_valid;
		echo '</p>';
	}
	xtc_db_close();
	exit;
}

if(isset($_REQUEST['check_connect']))
{
	$t_url = $_REQUEST['url'];
	$t_timeout = 5;
	$cc = MainFactory::create_object('ConnectChecker', array());
	try {
		if($_REQUEST['check_connect'] == 1) # check by GET
		{
			$connectinfo = $cc->check_connect($t_url, $t_timeout);
		}
		if($_REQUEST['check_connect'] == 2) # check by POST
		{
			$connectinfo = $cc->check_connect($t_url, $t_timeout, true, 'CONNECTION_TEST');
		}
		echo '<div class="connection_ok">'.$paypal->get_text('connection_ok').'</div>';
	}
	catch(Exception $e)
	{
		echo '<div class="connection_error">'.$paypal->get_text('connection_error').': '.$e->getMessage().'</div>';
	}

	xtc_db_close();
	exit;
}

function cleanColorValue($value) {
	$value = strtoupper(substr(trim($value), 0, 6));
	if(preg_match('/^[0-9A-F]{6}$/', $value) != 1) {
		return false;
	}
	return $value;
}

if($_SERVER['REQUEST_METHOD'] == 'POST') {
	isset($_SESSION['coo_page_token']) && $_SESSION['coo_page_token']->is_valid($_POST['page_token']);
	// save configuration
	foreach(array_keys($config) as $cfg_key) {
		if(isset($_POST[$cfg_key])) {
			if(strpos($cfg_key, 'color') !== false) {
				$cleancolor = cleanColorValue($_POST[$cfg_key]);
				if($cleancolor === false) {
					$_POST[$cfg_key] = '000000';
					$_SESSION[$messages_ns][] = $paypal->get_text('color_value_syntax_error');
				}
				else {
					$_POST[$cfg_key] = cleanColorValue($_POST[$cfg_key]);
				}
			}
			$paypal->setConfigValue($cfg_key, $_POST[$cfg_key]);
		}
	}
	$_SESSION[$messages_ns][] = $paypal->get_text('PPTXT_CONFIG_SAVED');

	xtc_redirect(PAGE_URL);
}

$messages = $_SESSION[$messages_ns];
$_SESSION[$messages_ns] = array();


$orders_status = xtc_get_orders_status();
ob_start();
?>
<!doctype HTML>
<html <?php echo HTML_PARAMS; ?>>
	<head>
		<meta http-equiv="Content-Type" content="text/html; charset=<?php echo $_SESSION['language_charset']; ?>">
		<title><?php echo TITLE; ?></title>
		<link rel="stylesheet" type="text/css" href="includes/stylesheet.css">
		<style>
			p.message {	margin: .5ex auto; background: rgb(240, 230, 140); border: 1px solid rgb(255, 0, 0); padding: 1em; }
			p.status { border: 1px solid; padding: 1ex 1em; }
			p.status_ok { background-color: rgb(152, 251, 152); border-color: rgb(34, 139, 34); }
			p.status_fail { background-color: rgb(244, 164, 96); border-color: rgb(165, 42, 42); }
			p.status_checking { background-color: rgb(247, 247, 133); border-color: rgb(234, 181, 23); }
			dl.adminform { position: relative; overflow: auto; }
			dl.adminform dd, dl.adminform dt { float: left; margin: .5ex 0; }
			dl.adminform dt { clear: left; width: 20%; }
			dl.adminform dd { width: 76%; }
			dl.adminform input[type="text"] { width: 99%; }
			dl.adminform input[type="text"].small { width: 5em; }
			dl.adminform input[type="radio"] { vertical-align: middle; margin: 2px; }
			input[type="submit"].btn_wide { width: auto; }
			.main fieldset { border: none; background: rgb(90%, 90%, 95%); margin: 1em auto; }
			.main legend { background: rgb(80%, 80%, 80%); padding: .5ex 1ex; }
			select { width: 20%; display: inline-block; vertical-align: top; }
			.select_info { margin: 0 1em; display: inline-block; width: 75%; }
			p.value_info { margin: 0; display: none; }
			a.apisig { font-size: 1em; display: inline-block; background: #bbb; padding: 3px 5px; box-shadow: 0 0 1px #000; }
			div.module_version { text-align: right; font-size: 0.8em; }
			button#start_connectcheck { display: block; width: 20em; margin: 2em auto; }
			table#connectcheck { background: #EEF2D0; display: none; width: 60%; margin: 2em auto; }
			div.connection_ok, div.connection_error { width: 100%; height: 100%; padding: .3ex; }
			div.connection_ok { background: #80C980; }
			div.connection_error { background: #FF9191; }

			form.bluegray {font-size: 0.9em; }
			form.bluegray fieldset {border: none; padding: 0; margin: 1ex 0 0 0; }
			form.bluegray legend {font-weight: bolder; font-size: 1.4em; background: #585858; color: #FFFFFF; padding: .2ex 0.5%; width: 99%; }
			form.bluegray dl.adminform {margin: 0; }
			form.bluegray dl.adminform dt, form.bluegray dl.adminform dd {line-height: 1.3; padding: 3px 0; margin: 0; }
			form.bluegray dl.adminform dt {width: 20%; float: left; font-weight: bold; padding: 2px;}
			form.bluegray dl.adminform dd {border-bottom: 1px dotted rgb(90, 90, 90); width: 78%; float: none; padding-left: 22%; background-color: #F7F7F7; }
			form.bluegray dl.adminform dd:nth-child(4n) {background: #D6E6F3; }
			dd.status_select { height: 4em; }
		</style>
	</head>
	<body>
		<!-- header //-->
		<?php require DIR_WS_INCLUDES . 'header.php'; ?>
		<!-- header_eof //-->

		<!-- body //-->
		<table border="0" width="100%" cellspacing="2" cellpadding="2">
			<tr>
				<td class="columnLeft2" width="<?php echo BOX_WIDTH; ?>" valign="top">
					<table border="0" width="<?php echo BOX_WIDTH; ?>" cellspacing="1" cellpadding="1" class="columnLeft">

						<!-- left_navigation //-->
						<?php require(DIR_WS_INCLUDES . 'column_left.php'); ?>
						<!-- left_navigation_eof //-->

					</table>
				</td>

				<!-- body_text //-->

				<td class="boxCenter" width="100%" valign="top">
					<table border="0" width="100%" cellspacing="0" cellpadding="0" class="">
						<tr>
							<td>
								<table border="0" width="100%" cellspacing="0" cellpadding="0">
									<tr>
										<td class="pageHeading" style="padding-left: 0px">##PPTXT_CONFIG_HEADING</td>
										<td width="80" rowspan="2">&nbsp;</td>
									</tr>
									<tr>
										<td class="main" valign="top">&nbsp;</td>
									</tr>
								</table>
							</td>
						</tr>
						<tr>
							<td class="main">
								<?php foreach($messages as $msg): ?>
								<p class="message"><?php echo $msg ?></p>
								<?php endforeach; ?>

								<div id="ppstatus">
									<p class="status status_checking">##PPTXT_CONFIG_CHECKING_CREDENTIALS</p>
								</div>

								<button id="start_connectcheck">##check_connection</button>

								<table id="connectcheck">
									<tr class="check">
										<td class="host">https://api-3t.sandbox.paypal.com/2.0</td>
										<td class="result">##checking_connectivity</td>
									</tr>
									<tr class="check">
										<td class="host">https://www.sandbox.paypal.com/cgi-bin/webscr</td>
										<td class="result">##checking_connectivity</td>
									</tr>
									<tr class="check">
										<td class="host">https://api-3t.paypal.com/2.0</td>
										<td class="result">##checking_connectivity</td>
									</tr>
									<!--
									<tr class="check">
										<td class="post_host">https://ipnpb.paypal.com/cgi-bin/webscr</td>
										<td class="result">##checking_connectivity</td>
									</tr>
									-->
									<tr class="check">
										<td class="host">https://www.paypal.com/cgi-bin/webscr</td>
										<td class="result">##checking_connectivity</td>
									</tr>
								</table>

								<form id="paypal_config" class="bluegray" action="<?php echo PAGE_URL ?>" method="POST">
									<input type="hidden" name="page_token" value="<?php echo $t_page_token ?>">
									<fieldset>
										<legend>##PPTXT_CONFIG_CREDS_LIVE</legend>
										<dl class="adminform">
											<dt><label for="username">##PPTXT_CONFIG_USERNAME</label></dt>
											<dd><input id="username" name="username" type="text" value="<?php echo $config['username'] ?>"></dd>
											<dt><label for="password">##PPTXT_CONFIG_PASSWORD</label></dt>
											<dd><input id="password" name="password" type="text" value="<?php echo $config['password'] ?>"></dd>
											<dt><label for="signature">##PPTXT_CONFIG_SIGNATURE</label></dt>
											<dd><input id="signature" name="signature" type="text" value="<?php echo $config['signature'] ?>"></dd>
											<dt><label for="get_live_apisig">##get_live_apisig</label></dt>
											<dd><a class="apisig" href="https://www.paypal.com/<?php echo $_SESSION['language_code'] ?>/cgi-bin/webscr?cmd=_get-api-signature&amp;generic-flow=true" target="_blank">##get_live_apisig_link</a></dd>
										</dl>
									</fieldset>
									<fieldset>
										<legend>##PPTXT_CONFIG_CREDS_SANDBOX</legend>
										<dl class="adminform">
											<dt><label for="username_sandbox">##PPTXT_CONFIG_USERNAME</label></dt>
											<dd><input id="username_sandbox" name="username_sandbox" type="text" value="<?php echo $config['username_sandbox'] ?>"></dd>
											<dt><label for="password_sandbox">##PPTXT_CONFIG_PASSWORD</label></dt>
											<dd><input id="password_sandbox" name="password_sandbox" type="text" value="<?php echo $config['password_sandbox'] ?>"></dd>
											<dt><label for="signature_sandbox">##PPTXT_CONFIG_SIGNATURE</label></dt>
											<dd><input id="signature_sandbox" name="signature_sandbox" type="text" value="<?php echo $config['signature_sandbox'] ?>"></dd>
											<dt><label for="get_live_apisig">##get_sandbox_apisig</label></dt>
											<dd><a class="apisig" href="https://www.sandbox.paypal.com/<?php echo $_SESSION['language_code'] ?>/cgi-bin/webscr?cmd=_get-api-signature&amp;generic-flow=true" target="_blank">##get_sandbox_apisig_link</a></dd>
										</dl>
									</fieldset>
									<fieldset>
										<legend>##PPTXT_CONFIG_SETTINGS</legend>
										<dl class="adminform">
											<dt>##PPTXT_CONFIG_SANDBOXMODE</dt>
											<dd>
												<input type="radio" name="sandbox_mode" value="1" id="sandbox_mode_1" <?php echo $config['sandbox_mode'] == 1 ? 'checked="checked"' : '' ?>>
													<label for="sandbox_mode_1">##PPTXT_CONFIG_ENABLED</label><br>
												<input type="radio" name="sandbox_mode" value="0" id="sandbox_mode_0" <?php echo $config['sandbox_mode'] == 0 ? 'checked="checked"' : '' ?>>
													<label for="sandbox_mode_0">##PPTXT_CONFIG_DISABLED</label><br>
											</dd>
											<dt>##PPTXT_CONFIG_USE_PAYPAL_CHECKOUT</dt>
											<dd>
												<input type="radio" name="use_paypal_checkout" value="1" id="use_paypal_checkout_1" <?php echo $config['use_paypal_checkout'] == 1 ? 'checked="checked"' : '' ?>>
													<label for="use_paypal_checkout_1">##PPTXT_CONFIG_ENABLED</label><br>
												<input type="radio" name="use_paypal_checkout" value="0" id="use_paypal_checkout_0" <?php echo $config['use_paypal_checkout'] == 0 ? 'checked="checked"' : '' ?>>
													<label for="use_paypal_checkout_0">##PPTXT_CONFIG_DISABLED</label><br>
											</dd>
											<dt>##PPTXT_CONFIG_USE_PAYPAL_PRODUCTS</dt>
											<dd>
												<input type="radio" name="use_paypal_products" value="1" id="use_paypal_products_1" <?php echo $config['use_paypal_products'] == 1 ? 'checked="checked"' : '' ?>>
													<label for="use_paypal_products_1">##PPTXT_CONFIG_ENABLED</label><br>
												<input type="radio" name="use_paypal_products" value="0" id="use_paypal_products_0" <?php echo $config['use_paypal_products'] == 0 ? 'checked="checked"' : '' ?>>
													<label for="use_paypal_products_0">##PPTXT_CONFIG_DISABLED</label><br>
											</dd>
											<dt>##PPTXT_CONFIG_BUTTON_STYLE</dt>
											<dd>
												<input type="radio" name="button_style" value="Sunrise" id="button_style_sunrise" <?php echo $config['button_style'] == 'Sunrise' ? 'checked="checked"' : '' ?>>
													<label for="button_style_sunrise">##PPTXT_CONFIG_BUTTON_STYLE_SUNRISE</label><br>
												<input type="radio" name="button_style" value="Silver" id="button_style_silver" <?php echo $config['button_style'] == 'Silver' ? 'checked="checked"' : '' ?>>
													<label for="button_style_silver">##PPTXT_CONFIG_BUTTON_STYLE_SILVER</label><br>
											</dd>
											<dt>##PPTXT_CONFIG_PREVENT_SELFPICKUP</dt>
											<dd>
												<input type="radio" name="prevent_selfpickup" value="prevent" id="prevent_selfpickup_prevent" <?php echo $config['prevent_selfpickup'] == 'prevent' ? 'checked="checked"' : '' ?>>
													<label for="prevent_selfpickup_prevent">##PPTXT_CONFIG_PREVENT_SELFPICKUP_PREVENT</label><br>
												<input type="radio" name="prevent_selfpickup" value="allow" id="prevent_selfpickup_allow" <?php echo $config['prevent_selfpickup'] == 'allow' ? 'checked="checked"' : '' ?>>
													<label for="prevent_selfpickup_allow">##PPTXT_CONFIG_PREVENT_SELFPICKUP_ALLOW</label><br>
												<div class="note">##selfpickup_note</div>
											</dd>
											<dt>##PPTXT_CONFIG_ORDERS_STATUS_ID_TMP</dt>
											<dd class="status_select">
												<select name="orders_status_id_tmp">
												<?php foreach($orders_status as $ostatus): ?>
													<option value="<?php echo $ostatus['id'] ?>" <?php echo $ostatus['id'] == $config['orders_status_id_tmp'] ? 'selected="selected"' : '' ?>>
														<?php echo $ostatus['text'] ?>
													</option>
												<?php endforeach ?>
												</select>
												<div class="select_info">
													##PPTXT_CONFIG_ORDERS_STATUS_ID_TMP_INFO
												</div>
											</dd>
											<dt>##PPTXT_CONFIG_ORDERS_STATUS_ID_PENDING</dt>
											<dd class="status_select">
												<select name="orders_status_id_pending">
												<?php foreach($orders_status as $ostatus): ?>
													<option value="<?php echo $ostatus['id'] ?>" <?php echo $ostatus['id'] == $config['orders_status_id_pending'] ? 'selected="selected"' : '' ?>>
														<?php echo $ostatus['text'] ?>
													</option>
												<?php endforeach ?>
												</select>
												<div class="select_info">
													##PPTXT_CONFIG_ORDERS_STATUS_ID_PENDING_INFO
												</div>
											</dd>
											<dt>##PPTXT_CONFIG_ORDERS_STATUS_ID</dt>
											<dd class="status_select">
												<select name="orders_status_id">
												<?php foreach($orders_status as $ostatus): ?>
													<option value="<?php echo $ostatus['id'] ?>" <?php echo $ostatus['id'] == $config['orders_status_id'] ? 'selected="selected"' : '' ?>>
														<?php echo $ostatus['text'] ?>
													</option>
												<?php endforeach ?>
												</select>
												<div class="select_info">
													##PPTXT_CONFIG_ORDERS_STATUS_ID_INFO
												</div>
											</dd>
											<dt>##PPTXT_CONFIG_ORDERS_STATUS_ID_AUTHORIZED</dt>
											<dd class="status_select">
												<select name="orders_status_id_authorized">
												<?php foreach($orders_status as $ostatus): ?>
													<option value="<?php echo $ostatus['id'] ?>" <?php echo $ostatus['id'] == $config['orders_status_id_authorized'] ? 'selected="selected"' : '' ?>>
														<?php echo $ostatus['text'] ?>
													</option>
												<?php endforeach ?>
												</select>
												<div class="select_info">
													##PPTXT_CONFIG_ORDERS_STATUS_ID_AUTHORIZED_INFO
												</div>
											</dd>
											<dt>##PPTXT_CONFIG_ORDERS_STATUS_ID_ORDERED</dt>
											<dd class="status_select">
												<select name="orders_status_id_ordered">
												<?php foreach($orders_status as $ostatus): ?>
													<option value="<?php echo $ostatus['id'] ?>" <?php echo $ostatus['id'] == $config['orders_status_id_ordered'] ? 'selected="selected"' : '' ?>>
														<?php echo $ostatus['text'] ?>
													</option>
												<?php endforeach ?>
												</select>
												<div class="select_info">
													##PPTXT_CONFIG_ORDERS_STATUS_ID_ORDERED_INFO
												</div>
											</dd>
											<dt>##PPTXT_CONFIG_PAYMENTACTION</dt>
											<dd class="status_select">
												<select name="paymentaction">
													<option value="Sale" <?php echo ($config['paymentaction'] == 'Sale') ? 'selected="selected"' : '' ?>>
														Sale
													</option>
													<option value="Authorization" <?php echo ($config['paymentaction'] == 'Authorization') ? 'selected="selected"' : '' ?>>
														Authorization
													</option>
													<option value="Order" <?php echo ($config['paymentaction'] == 'Order') ? 'selected="selected"' : '' ?>>
														Order
													</option>
												</select>
												<div class="select_info">
													<p class="value_info value_Sale">
														##PPTXT_CONFIG_PAYMENTACTIONINFO_SALE
													</p>
													<p class="value_info value_Authorization">
														##PPTXT_CONFIG_PAYMENTACTIONINFO_AUTHORIZATION
													</p>
													<p class="value_info value_Order">
														##PPTXT_CONFIG_PAYMENTACTIONINFO_ORDER
													</p>
													<dl class="adminform value_info value_Order">
														<dt><label for="order_max_auth">##pptxt_config_order_maxauth</label></dt>
														<dd>
															<input class="small" id="order_max_auth" name="order_max_auth" type="text" value="<?php echo $config['order_max_auth'] ?>">
														</dd>
														<dt><label for="order_max_capture">##pptxt_config_order_maxcapture</label></dt>
														<dd>
															<input class="small" id="order_max_capture" name="order_max_capture" type="text" value="<?php echo $config['order_max_capture'] ?>">
														</dd>
													</dl>
												</div>
											</dd>

										</dl>
									</fieldset>
									<fieldset>
										<legend>##pptxt_config_appearance</legend>
										<dl class="adminform">
											<!--
											<dt><label for="cppheaderimage">##cppheaderimage</label></dt>
											<dd>
												<input id="cppheaderimage" name="cppheaderimage" type="text" value="<?php echo $config['cppheaderimage'] ?>">
											</dd>
											-->
											<dt><label for="cpplogoimage">##cpplogoimage</label></dt>
											<dd>
												<input id="cpplogoimage" name="cpplogoimage" type="text" value="<?php echo $config['cpplogoimage'] ?>">
												<br>
												##note_logo_url_text
											</dd>
											<!--
											<dt><label for="cppheaderbordercolor">##cppheaderbordercolor</label></dt>
											<dd>
												<input id="cppheaderbordercolor" name="cppheaderbordercolor" type="text" value="<?php echo $config['cppheaderbordercolor'] ?>">
											</dd>
											<dt><label for="cppheaderbackcolor">##cppheaderbackcolor</label></dt>
											<dd>
												<input id="cppheaderbackcolor" name="cppheaderbackcolor" type="text" value="<?php echo $config['cppheaderbackcolor'] ?>">
											</dd>
											-->
											<dt><label for="cpppayflowcolor">##cpppayflowcolor</label></dt>
											<dd>
												<input id="cpppayflowcolor" name="cpppayflowcolor" type="text" value="<?php echo $config['cpppayflowcolor'] ?>">
											</dd>
											<dt><label for="cppcartbordercolor">##cppcartbordercolor</label></dt>
											<dd>
												<input id="cppcartbordercolor" name="cppcartbordercolor" type="text" value="<?php echo $config['cppcartbordercolor'] ?>">
											</dd>
											<dt><label for="pagestyle">##pagestyle</label></dt>
											<dd>
												<input id="pagestyle" name="pagestyle" type="text" value="<?php echo $config['pagestyle'] ?>">
											</dd>
											<dt><label for="brandname">##brandname</label></dt>
											<dd>
												<input id="brandname" name="brandname" type="text" value="<?php echo $config['brandname'] ?>">
											</dd>
									</fieldset>
									<input class="button btn_wide" type="submit" value="##PPTXT_CONFIG_SAVE">
								</form>

								<div class="module_version"><?php echo GMPayPal::MODULE_VERSION ?></div>
							</td>
						</tr>
					</table>
				</td>

				<!-- body_text_eof //-->
				
			</tr>
		</table>
		<!-- body_eof //-->

		<script>
		$(function() {
			$('select').change(function(e) {
				var value = $(this).val();
				$('.value_info', $(this).parent()).hide();
				$('.value_'+value, $(this).parent()).show();
			});
			$('select').change();

			$('#ppstatus').load('<?php echo PAGE_URL.'?'.http_build_query(array('check_credentials' => '1')) ?>');

			$('body').delegate('#start_connectcheck', 'click', function() {
				$(this).hide();
				$('#connectcheck').show();
				$('#connectcheck tr.check').each(function() {
					var url = $('td.host', this).text();
					var post_url = $('td.post_host', this).text();
					if(url) {
						$('td.result', this).load('<?php echo PAGE_URL ?>?check_connect=1&url='+url);
					}
					if(post_url) {
						$('td.result', this).load('<?php echo PAGE_URL ?>?check_connect=2&url='+url);
					}
				});
			});

			$('body').delegate('div.module_version', 'dblclick', function() { $('form#paypal_config').toggleClass('bluegray'); });
		});
		</script>

		<!-- footer //-->
		<?php require DIR_WS_INCLUDES . 'footer.php'; ?>
		<!-- footer_eof //-->
	</body>
</html>
<?php
require DIR_WS_INCLUDES . 'application_bottom.php';

$content = ob_get_clean();
$content = $paypal->replaceLanguagePlaceholders($content);
echo $content;
