<?php
/* --------------------------------------------------------------
	GMPayPalGuestAccountManager.php 2014-09-04 mabr
	Gambio GmbH
	http://www.gambio.de
	Copyright (c) 2013 Gambio GmbH
	Released under the GNU General Public License (Version 2)
	[http://www.gnu.org/licenses/gpl-2.0.html]
	--------------------------------------------------------------
*/

require_once DIR_FS_CATALOG.'inc/xtc_create_password.inc.php';
require_once DIR_FS_CATALOG.'inc/xtc_encrypt_password.inc.php';

class GMPayPalGuestAccountManager {
	public function __construct() {

	}

	public function createAccount(GetExpressCheckoutDetailsResponseType $gecd_response) {
		if(isset($_SESSION['customer_id']) && $_SESSION['customer_id'] > 0) {
			throw new GMPayPalGuestAccountManagerUserAlreadyLoggedInException();
		}

		$customers_status = DEFAULT_CUSTOMERS_STATUS_ID_GUEST;
		$password = xtc_create_password(8);
		$vat = '';
		$cid = 'ppguest_'.$this->transcodeInbound((string)$gecd_response->GetExpressCheckoutDetailsResponseDetails->PayerInfo->PayerID);
		$firstname = $this->transcodeInbound((string)$gecd_response->GetExpressCheckoutDetailsResponseDetails->PayerInfo->PayerName->FirstName);
		$lastname = $this->transcodeInbound((string)$gecd_response->GetExpressCheckoutDetailsResponseDetails->PayerInfo->PayerName->LastName);
		$email = $this->transcodeInbound((string)$gecd_response->GetExpressCheckoutDetailsResponseDetails->PayerInfo->Payer);
		$phone = $this->transcodeInbound((string)$gecd_response->GetExpressCheckoutDetailsResponseDetails->PayerInfo->Address->Phone);
		$street_address = $this->transcodeInbound((string)$gecd_response->GetExpressCheckoutDetailsResponseDetails->PayerInfo->Address->Street1);
		$street_address2 = $this->transcodeInbound((string)$gecd_response->GetExpressCheckoutDetailsResponseDetails->PayerInfo->Address->Street2);
		$postcode = $this->transcodeInbound((string)$gecd_response->GetExpressCheckoutDetailsResponseDetails->PayerInfo->Address->PostalCode);
		$city = $this->transcodeInbound((string)$gecd_response->GetExpressCheckoutDetailsResponseDetails->PayerInfo->Address->CityName);
		$country_iso2 = $this->transcodeInbound((string)$gecd_response->GetExpressCheckoutDetailsResponseDetails->PayerInfo->Address->Country);
		$country = $this->_findCountryIdByIso2($country_iso2);
		if(empty($street_address2) !== true)
		{
			$company = $street_address2;
		}
		else
		{
			$company = '';
		}

		$sql_data_array = array(
			'customers_cid' => $cid,
			'customers_vat_id' => $vat,
			'customers_vat_id_status' => 0,
			'customers_status' => $customers_status,
			'customers_firstname' => $firstname,
			'customers_lastname' => $lastname,
			'customers_email_address' => $email,
			'customers_telephone' => $phone,
			'customers_fax' => '',
			'customers_newsletter' => '',
			'account_type' => '1',
			'customers_password' => xtc_encrypt_password($password),
			'customers_gender' => 'n',
			'customers_dob' => '',
		);

		xtc_db_perform(TABLE_CUSTOMERS, $sql_data_array);

		$_SESSION['pp_cid'] = $cid;
		$_SESSION['account_type'] = '1';
		$_SESSION['customer_id'] = xtc_db_insert_id();

		$sql_data_array = array(
			'customers_id' => $_SESSION['customer_id'],
			'entry_firstname' => $firstname,
			'entry_lastname' => $lastname,
			'entry_street_address' => $street_address,
			'entry_postcode' => $postcode,
			'entry_city' => $city,
			'entry_country_id' => $country,
			'entry_company' => $company,
			'entry_suburb' => '',
			'entry_zone_id' => '',
			'entry_state' => '',
			'address_class' => 'paypal_guest',
		);

		xtc_db_perform(TABLE_ADDRESS_BOOK, $sql_data_array);
		$address_id = xtc_db_insert_id();

		$t_update_customer_query =
			'UPDATE
				customers
			SET
				customers_cid = \':cid\',
				customers_default_address_id = \':address_id\'
			WHERE
				customers_id = \':customers_id\'';
		$t_update_customer_query = strtr($t_update_customer_query,
				array(
					':cid' => $cid,
					':address_id' => $address_id,
					':customers_id' => (int)$_SESSION['customer_id'],
				)
			);
		xtc_db_query($t_update_customer_query);

		$t_customers_info_query =
			'INSERT INTO
				customers_info (customers_info_id, customers_info_number_of_logons, customers_info_date_account_created)
			VALUES
				(\':customers_id\', \'0\', NOW())';
		$t_customers_info_query = strtr($t_customers_info_query,
				array(
					':customers_id' => (int)$_SESSION['customer_id'],
				)
			);
		xtc_db_query($t_customers_info_query);

		$_SESSION['customer_first_name'] = $firstname;
		$_SESSION['customer_last_name'] = $lastname;
		$_SESSION['customer_default_address_id'] = $address_id;
		$_SESSION['customer_country_id'] = $country;
		$_SESSION['customer_zone_id'] = $zone_id;
		$_SESSION['customer_vat_id'] = $vat;

	}

	protected function _findCountryIdByIso2($iso2) {
		$query = "SELECT * FROM countries WHERE countries_iso_code_2 = ':iso2'";
		$query = strtr($query, array(':iso2' => $iso2));
		$countries_id = 0;
		$result = xtc_db_query($query);
		while($row = xtc_db_fetch_array($result)) {
			$countries_id = $row['countries_id'];
		}
		return $countries_id;
	}


	/* === auxilliary functions === */

	protected function _localEncodingIsLatin1() {
		$is_latin1 = strpos(strtolower($_SESSION['language_charset']), 'iso-8859-1') !== false;
		return $is_latin1;
	}

	/** transcode incoming string from UTF-8 to whatever the shop system is currenty using */
	public function transcodeInbound($string) {
		if($this->_localEncodingIsLatin1()) {
			$output = utf8_decode($string);
		}
		else {
			$output = $string;
		}
		return $output;
	}

}


class GMPayPalGuestAccountManagerException extends Exception {}
class GMPayPalGuestAccountManagerUserAlreadyLoggedInException extends GMPayPalGuestAccountManagerException {}
