<?php
/* --------------------------------------------------------------
	GMPayPalIPNProcessor.php 2.5_2014-08-11_1722 mabr
	Gambio GmbH
	http://www.gambio.de
	Copyright (c) 2014 Gambio GmbH
	Released under the GNU General Public License (Version 2)
	[http://www.gnu.org/licenses/gpl-2.0.html]
	--------------------------------------------------------------
*/

class GMPayPalIPNProcessor {
	protected $_paypal;
	protected $_sandbox_mode;
	protected $_logger;

	protected $_ipn_fields = array(
		'address_city', 'address_country', 'address_country_code', 'address_name', 'address_state', 'address_status', 'address_street', 'address_zip',
		'business', 'custom', 'first_name', 'invoice', 'item_name', 'item_number', 'last_name', 'mc_currency', 'mc_fee', 'mc_gross', 'mc_handling', 'mc_shipping',
		'notify_version', 'parent_txn_id', 'payer_email', 'payer_id', 'payer_status', 'payment_date', 'payment_status', 'payment_type', 'quantity', 'reason_code', 'receipt_ID',
		'receiver_email', 'receiver_id', 'residence_country', 'shipping', 'tax', 'test_ipn', 'txn_id', 'txn_type', 'verify_sign', 'pending_reason'
	);


	const LOGLEVEL_INFO = 0;
	const LOGLEVEL_WARN = 10;
	const LOGLEVEL_ERROR = 20;
 	const LOGLEVEL_DEBUG = 99;

	public function __construct() {
		$this->_paypal = new GMPayPal();
		$ppconfig = $this->_paypal->getConfigArray();
		$this->_sandbox_mode = $ppconfig['sandbox_mode'] == true;
		$this->_logger = LogControl::get_instance();
	}

	protected function _log($message, $loglevel = self::LOGLEVEL_INFO) {
		if($this->_logger !== false) {
			switch($loglevel)
			{
				case self::LOGLEVEL_ERROR:
					$this->_logger->error($message, 'payment', 'payment.paypal-ipn');
					break;
				case self::LOGLEVEL_WARN:
					$this->_logger->warning($message, 'payment', 'payment.paypal-ipn');
					break;
				case self::LOGLEVEL_INFO:
				case self::LOGLEVEL_DEBUG:
				default:
					$this->_logger->notice($message, 'payment', 'payment.paypal-ipn');
					break;
			}
		}
	}

	public function process($post, $p_raw_input) {
		$this->_log("RAW input:\n".$t_raw_input);
		$this->_log("Received IPN data:\n".print_r($post, true));
		$this->_log("Sending acknowledgement back to PayPal.");
		//$acknowledgement = $this->_sendAcknowledgement($post);
		$acknowledgement = $this->_sendRawAcknowledgement($p_raw_input);
		if($acknowledgement == 'VERIFIED') {
			// valid Instant Notification Message
			$this->_log('IPN acknowledged');
			// further checking?
			if(empty($post['txn_id'])) {
				$this->_log('IPN without txn_id, aborting');
				$result = false;
			}
			else {
				$t_orders_id = $this->_getOrdersIdByTransactionId($post['txn_id']);
				if(empty($t_orders_id) === false)
				{
					$this->_log('corresponding orders_id is '.$t_orders_id);
					# update CheckoutDetails, if possible
					try {
						$this->_paypal->getExpressCheckoutDetails((int)$t_orders_id, true);
					}
					catch(Exception $e)
					{
						$this->_log('FAILED to retrieve CheckoutDetails (token too old?)');
					}
				}
				if(!$this->_txnLogged($post['txn_id'], $post['payment_status'])) {
					// this is a new IPN
					$this->_saveIPN($post);
					$this->_log('IPN saved');
				}
				else {
					// we've seen this IPN before
					$this->_saveIPN($post);
					$this->_log('IPN saved (duplicate)');
				}
				$result = true;
			}
		}
		else {
			$this->_log('IPN did NOT get acknowledged!');
			$result = false;
		}
		return $result;
	}

	protected function _saveIPN($post) {
		$data = array();
		foreach($this->_ipn_fields as $fieldname) {
			if(isset($post[$fieldname])) {
				$data[$fieldname] = xtc_db_input(stripslashes($post[$fieldname]));
			}
			else {
				$data[$fieldname] = '';
			}
		}
		$data['received_time'] = 'now()';
		$data['ipn_raw'] = serialize($post);
		xtc_db_perform('paypal_ipn', $data, 'insert');
		// update payment_status
		$query = "UPDATE paypal_transactions SET paymentstatus = '".xtc_db_input($post['payment_status'])."' WHERE transaction_id = '".xtc_db_input($post['txn_id'])."'";

		// update order status
		$ppconfig = $this->_paypal->getConfigArray();
		switch(strtolower($post['payment_status'])) {
			case 'completed':
				$new_orders_status = $ppconfig['orders_status_id'];
				$new_orders_status_message = $this->_paypal->get_text('ipn_transaction_completed');
				break;
			case 'pending':
				$new_orders_status = $ppconfig['orders_status_id_pending'];
				$new_orders_status_message = $this->_paypal->get_text('ipn_transaction_pending');
				break;
			default:
				$new_orders_status = false;
		}
		if($new_orders_status !== false) {
			$this->_updateOrdersStatus($post['txn_id'], $new_orders_status, $new_orders_status_message);
		}

		xtc_db_query($query);
	}

	protected function _updateOrdersStatus($txn_id, $orders_status_id, $comment) {
		$orders_id = $this->_getOrdersIdByTransactionId($txn_id);
		if($orders_id !== false) {
			$orders_query = "UPDATE orders SET orders_status = ".(int)$orders_status_id." WHERE orders_id = ".(int)$orders_id;
			xtc_db_query($orders_query);
			$osh_query = "INSERT INTO orders_status_history (`orders_id`, `orders_status_id`, `date_added`, `customer_notified`, `comments`) ".
				"VALUES (:orders_id, :orders_status_id, NOW(), 0, ':comments')";
			$osh_query = strtr($osh_query, array(
				':orders_id' => (int)$orders_id,
				':orders_status_id' => (int)$orders_status_id,
				':comments' => xtc_db_input($comment)));
			xtc_db_query($osh_query);
		}
		else {
			$this->_log('WARNING: no order for transaction_id '.$txn_id);
		}
	}

	protected function _getOrdersIdByTransactionId($txn_id) {
		$query = "SELECT orders_id FROM paypal_transactions WHERE transaction_id = ':txn_id' ORDER BY `timestamp` ASC";
		$query = strtr($query, array(':txn_id' => xtc_db_input($txn_id)));
		$result = xtc_db_query($query);
		$orders_id = false;
		while($row = xtc_db_fetch_array($result)) {
			$orders_id = $row['orders_id'];
		}
		return $orders_id;
	}

	protected function _txnLogged($txn_id, $payment_status) {
		/*
		$query = "SELECT paypal_ipn_id FROM paypal_ipn WHERE txn_id = ':txn_id'";
		$query = strtr($query, array(':txn_id' => xtc_db_input($txn_id)));
		$result = xtc_db_query($query);
		$found = xtc_db_num_rows($result) > 0;
		*/
		$query = "SELECT paymentstatus FROM paypal_transactions WHERE transaction_id = '".xtc_db_input($txn_id)."'";
		$result = xtc_db_query($query);
		$old_payment_status = false;
		while($row = xtc_db_fetch_array($result)) {
			$old_payment_status = $row['paymentstatus'];
		}
		$found = strcmp($old_payment_status, $payment_status) == 0;
		return $found;
	}

	public function logNonPostRequest() {
		$remote_ip = $_SERVER['REMOTE_ADDR'];
		$this->_log("Received invalid request from $remote_ip (not POST)");
	}

	protected function _sendAcknowledgement($post) {
		//return $this->_sendAcknowledgementByFsock($post);
		return $this->_sendAcknowledgementByCurl($post);
	}

	/** This is PayPal's reference implementation (adapted) */
	protected function _sendAcknowledgementByFsock($post) {
		$req = 'cmd=_notify-validate';

		foreach($post as $key => $value) {
			$value = urlencode(stripslashes($value));
			$req .= "&$key=$value";
		}
		if($this->_sandbox_mode) {
			$url = 'ssl://www.sandbox.paypal.com';
			$host = 'www.sandbox.paypal.com';
		}
		else {
			$url = 'ssl://ipnpb.paypal.com';
			$host = 'ipnpb.paypal.com';
		}
		$header .= "POST /cgi-bin/webscr HTTP/1.0\r\n";
		$header .= "Host: $host:443\r\n";
		$header .= "Content-Type: application/x-www-form-urlencoded\r\n";
		$header .= "Content-Length: " . strlen($req) . "\r\n\r\n";

		$fp = fsockopen($url, 443, $errno, $errstr, 30);

		// Post request back to PayPal for validation
		fputs($fp, $header . $req);
		$res = '';
		while(!feof($fp)) {
			$res .= fgets($fp, 1024);
		}
		fclose($fp);
		$res = trim($res);
		$this->_log('ACK result: '.$res);
		return $res;
	}

	protected function _sendAcknowledgementByCurl($post) {
		$post['cmd'] = '_notify-validate';
		$post_out = $post;
		if(get_magic_quotes_gpc() == true)
		{
			$post_out = array();
			foreach($post as $key => $value) {
				$post_out[$key] = stripslashes($value);
			}
		}
		if($this->_sandbox_mode || isset($post['test_ipn'])) {
			$url = 'https://www.sandbox.paypal.com/cgi-bin/webscr';
		}
		else {
			$url = 'https://www.paypal.com/cgi-bin/webscr';
		}
		$this->_log("Sending back IPN data to $url:\n".print_r($post_out, true));
		$ch = curl_init();
		curl_setopt_array($ch, array(
			CURLOPT_URL => $url,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_TIMEOUT => 10,
			CURLOPT_POSTFIELDS => $post_out,
		));
		$res = curl_exec($ch);
		curl_close($ch);
		$res = trim($res);
		$this->_log('ACK result: '.$res);
		return $res;
	}

	protected function _sendRawAcknowledgement($p_raw_input)
	{
		if($this->_sandbox_mode) {
			$url = 'https://www.sandbox.paypal.com/cgi-bin/webscr';
		}
		else {
			$url = 'https://www.paypal.com/cgi-bin/webscr';
		}
		$t_out_data = $p_raw_input .= '&cmd=_notify-validate';
		$ch = curl_init();
		curl_setopt_array($ch, array(
			CURLOPT_URL => $url,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_TIMEOUT => 10,
			CURLOPT_POSTFIELDS => $t_out_data,
		));
		$res = curl_exec($ch);
		curl_close($ch);
		$res = trim($res);
		$this->_log('ACK result: '.$res);
		return $res;
	}
}
