<?php
/* --------------------------------------------------------------
  TopNavigationBoxContentView.inc.php 2014-07-17 gambio
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2014 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------
 */

require_once(DIR_FS_CATALOG . 'gm/inc/gm_prepare_number.inc.php');

class TopNavigationBoxContentView extends ContentView
{
	protected $coo_xtc_price;
	
	public function __construct()
	{
		parent::__construct();
		$this->set_content_template('boxes/box_top_navigation.html');
		$this->set_caching_enabled(false);
	}
	
	protected function set_validation_rules()
	{
		// SET VALIDATION RULES
		$this->validation_rules_array['coo_xtc_price']	= array('type' => 'object',
																'object_type' => 'xtcPrice');
	}

	public function prepare_data()
	{
		$t_show_arrow = 0;

		$this->coo_xtc_price=  new xtcPrice($_SESSION['currency'], $_SESSION['customers_status']['customers_status_id']);

		$this->content_array['HOME_URL'] = xtc_href_link(FILENAME_DEFAULT, '', 'NONSSL');

		if(isset($_SESSION['customer_id']) == false)
		{
			$this->content_array['LOGIN_URL'] = xtc_href_link(FILENAME_LOGIN, '', 'NONSSL');
		}
		else
		{
			$this->content_array['LOGOFF_URL'] = xtc_href_link(FILENAME_LOGOFF, '', 'NONSSL');
			$this->content_array['ACCOUNT_URL'] = xtc_href_link(FILENAME_ACCOUNT, '', 'NONSSL');
		}

		$t_gm_show_wishlist = gm_get_conf('GM_SHOW_WISHLIST');
		if($t_gm_show_wishlist == 'true')
		{
			$this->content_array['WISHLIST_URL'] = xtc_href_link(FILENAME_WISHLIST, '', 'NONSSL');
		}

		$this->content_array['CART_URL'] = xtc_href_link(FILENAME_SHOPPING_CART, '', 'NONSSL');

		if(count($this->coo_xtc_price->currencies) > 1 && gm_get_conf('SHOW_TOP_CURRENCY_SELECTION') == 'true' && strpos(gm_get_env_info('SCRIPT_NAME'), 'checkout') === false)
		{
			$this->content_array['CURRENT_CURRENCY'] = $_SESSION['currency'];
		}

		if(is_array($this->content_array['customers_data']) == false)
		{
			$this->content_array['customers_data'] = array();
		}

		$this->content_array['customers_data']['FIRST_NAME'] = '';
		if(isset($_SESSION['customer_first_name']))
		{
			$this->content_array['customers_data']['FIRST_NAME'] = $_SESSION['customer_first_name'];
		}

		$this->content_array['customers_data']['LAST_NAME'] = '';
		if(isset($_SESSION['customer_last_name']))
		{
			$this->content_array['customers_data']['LAST_NAME'] = $_SESSION['customer_last_name'];
		}

		$this->content_array['customers_data']['PRODUCTS_DISCOUNT'] = '';
		if((double)$_SESSION['customers_status']['customers_status_discount'] > 0)
		{
			$t_show_arrow = 1;
			$this->content_array['customers_data']['PRODUCTS_DISCOUNT'] = gm_prepare_number($_SESSION['customers_status']['customers_status_discount'], ',');
		}

		$this->content_array['customers_data']['ORDER_DISCOUNT'] = '';
		if((double)$_SESSION['customers_status']['customers_status_ot_discount'] > 0 && $_SESSION['customers_status']['customers_status_ot_discount_flag'] == '1')
		{
			$t_show_arrow = 1;
			$this->content_array['customers_data']['ORDER_DISCOUNT'] = gm_prepare_number($_SESSION['customers_status']['customers_status_ot_discount'], ',');
		}

		$this->content_array['customers_data']['PUBLIC'] = $_SESSION['customers_status']['customers_status_public'];

		$this->content_array['customers_data']['MIN_ORDER'] = '';
		if((double)$_SESSION['customers_status']['customers_status_min_order'] > 0)
		{
			$t_show_arrow = 1;
			$t_min_order = $this->coo_xtc_price->xtcFormat($_SESSION['customers_status']['customers_status_min_order'], true);
			$this->content_array['customers_data']['MIN_ORDER'] = $t_min_order;
		}

		$this->content_array['customers_data']['MAX_ORDER'] = '';
		if((double)$_SESSION['customers_status']['customers_status_max_order'] > 0)
		{
			$t_show_arrow = 1;
			$t_max_order = $this->coo_xtc_price->xtcFormat($_SESSION['customers_status']['customers_status_max_order'], true);
			$this->content_array['customers_data']['MAX_ORDER'] = $t_max_order;
		}

		$this->content_array['customers_data']['GENDER'] = $_SESSION['customer_gender'];

		$this->content_array['customers_data']['GROUP'] = $_SESSION['customers_status']['customers_status_name'];

		$this->content_array['customers_data']['ICON'] = '';
		if(file_exists('admin/images/icons/' . basename($_SESSION['customers_status']['customers_status_image'])))
		{
			$this->content_array['customers_data']['ICON'] = 'admin/images/icons/' . basename($_SESSION['customers_status']['customers_status_image']);
		}

		$this->content_array['customers_data']['SHOW_ARROW'] = $t_show_arrow;

		if(gm_get_conf('SHOW_TOP_LANGUAGE_SELECTION') == 'true')
		{
			$t_language_code = $_SESSION['language_code'];

			# workaround if session language data is incomplete
			if(empty($_SESSION['language']) == false && empty($t_language_code))
			{
				$c_directory = gm_prepare_string($_SESSION['language']);

				$t_query = "SELECT 
								code
							FROM 
								" . TABLE_LANGUAGES . "
							WHERE 
								directory = '" . $c_directory . "'
							LIMIT 1";
				$t_result = xtc_db_query($t_query);
				if(xtc_db_num_rows($t_result) == 1)
				{
					$t_language_data_array = xtc_db_fetch_array($t_result);
					$t_language_code = $t_language_data_array['code'];
				}
			}

			$this->content_array['LANGUAGE_ICON'] = 'lang/' . basename($_SESSION['language']) . '/' . basename($t_language_code) . '.png';
		}

		# topmenu content
		$coo_content = MainFactory::create_object('ContentBoxContentView');
		$coo_content->get_html('topmenu_corner'); #init internal variables. do nothing with return value
		$t_content_array = $coo_content->get_content_array();

		if(is_array($t_content_array['CONTENT_LINKS_DATA']))
		{
			$this->content_array['CONTENT_LINKS_DATA'] = $t_content_array['CONTENT_LINKS_DATA'];
		}
	}
}