<?php
/* --------------------------------------------------------------
  AddressBookDetailsContentView.inc.php 2014-04-24 gm
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2014 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------


  based on:
  (c) 2000-2001 The Exchange Project  (earlier name of osCommerce)
  (c) 2002-2003 osCommerce(address_book_process.php,v 1.77 2003/05/27); www.oscommerce.com
  (c) 2003	 nextcommerce (address_book_process.php,v 1.13 2003/08/17); www.nextcommerce.org
  (c) 2003 XT-Commerce - community made shopping http://www.xt-commerce.com ($Id: address_book_process.php 1218 2005-09-16 11:38:37Z mz $)

  Released under the GNU General Public License
  --------------------------------------------------------------------------------------- */

// include needed functions
include_once('inc/xtc_get_zone_name.inc.php');
include_once('inc/xtc_get_country_list.inc.php');

class AddressBookDetailsContentView extends ContentView
{
	protected $action_edit;
	protected $process;
	protected $coo_address;
	protected $customer_country_id;
	protected $entry_state_has_zones;
	protected $customer_default_address_id;
	protected $error_array = array();

	function __construct()
	{
		parent::__construct();
		
		$this->set_content_template('module/address_book_details.html');
		$this->set_flat_assigns(true);
		$this->set_caching_enabled(false);
	}

	protected function set_validation_rules()
	{
		$this->validation_rules_array['action_edit']					= array('type' => 'bool');
		$this->validation_rules_array['process']						= array('type' => 'bool');
		$this->validation_rules_array['entry_state_has_zones']			= array('type' => 'bool');
		$this->validation_rules_array['customer_country_id']			= array('type' => 'int');
		$this->validation_rules_array['customer_default_address_id']	= array('type' => 'int');
		$this->validation_rules_array['error_array']					= array('type' => 'array');
		$this->validation_rules_array['coo_address']					= array('type' => 'object',
																				'object_type' => 'AddressModel');
	}
	
	protected function set_deprecated_array()
	{
		$this->deprecated_array[1][] = 'BUTTON_BACK';
		$this->deprecated_array[1][] = 'BUTTON_UPDATE';
		$this->deprecated_array[1][] = 'INPUT_MALE';
		$this->deprecated_array[1][] = 'INPUT_FEMALE';
		$this->deprecated_array[1][] = 'INPUT_FIRSTNAME';
		$this->deprecated_array[1][] = 'INPUT_LASTNAME';
		$this->deprecated_array[1][] = 'INPUT_COMPANY';
		$this->deprecated_array[1][] = 'INPUT_STREET';
		$this->deprecated_array[1][] = 'INPUT_SUBURB';
		$this->deprecated_array[1][] = 'INPUT_CODE';
		$this->deprecated_array[1][] = 'INPUT_CITY';
		$this->deprecated_array[1][] = 'INPUT_STATE';
		$this->deprecated_array[1][] = 'SELECT_COUNTRY';
		$this->deprecated_array[1][] = 'CHECKBOX_PRIMARY';
	}

	function prepare_data()
	{
		$t_uninitialized_array = $this->get_uninitialized_variables(array('customer_default_address_id',
																		  'customer_country_id',
																		  'entry_state_has_zones',
																		  'coo_address')
		);

		if(empty($t_uninitialized_array))
		{
			// ADD ERROR MESSAGES
			$this->add_error_messages();

			// ADD FORM DATA
			$this->add_data();
		}
		else
		{
			trigger_error("Variable(s) " . implode(', ', $t_uninitialized_array) . " do(es) not exist in class " . get_class($this) . " or is/are null", E_USER_ERROR);
		}
	}

	protected function add_error_messages()
	{
		if(is_array($this->error_array) && empty($this->error_array) == false)
		{
			foreach($this->error_array AS $t_error => $t_error_text)
			{
				$this->content_array[$t_error] = $t_error_text;
				$GLOBALS['messageStack']->add('address_book_details', $t_error_text);
			}
		}
	}

	protected function add_data()
	{
		if(is_array($this->content_array['form_data']) == false)
		{
			$this->content_array['form_data'] = array();
		}
		
		$this->add_buttons();
		$this->add_gender();
		$this->add_firstname();
		$this->add_lastname();
		$this->add_company();
		$this->add_street_address();
		$this->add_suburb();
		$this->add_postcode();
		$this->add_city();
		$this->add_state();
		$this->add_country();
		$this->add_privacy();
		$this->add_primary_address();
	}

	protected function add_buttons()
	{
		$t_back_link = xtc_href_link(FILENAME_ADDRESS_BOOK, '', 'SSL');
		$this->content_array['BUTTON_BACK'] = '<a href="' . $t_back_link . '">' . xtc_image_button('button_back.gif', IMAGE_BUTTON_BACK) . '</a>';
		$this->content_array['BUTTON_BACK_LINK'] = $t_back_link;
		$this->content_array['HIDDEN_FIELD_NAME'] = 'action';
		if(isset($this->action_edit) && $this->action_edit)
		{
			$this->content_array['BUTTON_UPDATE'] = xtc_draw_hidden_field('action', 'update') . xtc_image_submit('button_continue.gif', IMAGE_BUTTON_CONTINUE);
			$this->content_array['HIDDEN_FIELD_VALUE'] = 'update';
		}
		else
		{
			$this->content_array['BUTTON_UPDATE'] = xtc_draw_hidden_field('action', 'process') . xtc_image_submit('button_continue.gif', IMAGE_BUTTON_CONTINUE);
			$this->content_array['HIDDEN_FIELD_VALUE'] = 'process';
		}
	}

	protected function add_gender()
	{
		if(ACCOUNT_GENDER == 'true')
		{
			$t_male = ($this->coo_address->get_('entry_gender') == 'm') ? true : false;
			$t_female = ($this->coo_address->get_('entry_gender') == 'f') ? true : false;

			$this->content_array['gender'] = '1';
			$this->content_array['INPUT_MALE'] = xtc_draw_radio_field(array('name' => 'gender', 'suffix' => MALE . '&nbsp;'), 'm', $t_male);
			$this->content_array['INPUT_FEMALE'] = xtc_draw_radio_field(array('name' => 'gender', 'suffix' => FEMALE . '&nbsp;', 'text' => (xtc_not_null(ENTRY_GENDER_TEXT) ? '<span class="inputRequirement">&nbsp;' . ENTRY_GENDER_TEXT . '</span>' : '')), 'f', $t_female);

			if(is_array($this->content_array['form_data']['gender']) == false)
			{
				$this->content_array['form_data']['gender'] = array();
			}
			$this->content_array['form_data']['gender']['name'] = 'gender';
			if(is_array($this->content_array['form_data']['gender']['m']) == false)
			{
				$this->content_array['form_data']['gender']['m'] = array();
			}
			$this->content_array['form_data']['gender']['m']['value'] = 'm';
			$this->content_array['form_data']['gender']['m']['checked'] = '0';
			if(is_array($this->content_array['form_data']['gender']['f']) == false)
			{
				$this->content_array['form_data']['gender']['f'] = array();
			}
			$this->content_array['form_data']['gender']['f']['value'] = 'f';
			$this->content_array['form_data']['gender']['f']['checked'] = '0';

			if($t_male)
			{
				$this->content_array['form_data']['gender']['m']['checked'] = '1';
			}
			if($t_female)
			{
				$this->content_array['form_data']['gender']['f']['checked'] = '1';
			}

			$this->content_array['form_data']['gender']['required'] = 1;
			$this->content_array['form_data']['gender']['required_symbol'] = ENTRY_GENDER_TEXT;
		}
		else
		{
			$this->content_array['gender'] = '0';
		}
	}

	protected function add_firstname()
	{
		$this->content_array['INPUT_FIRSTNAME'] = xtc_draw_input_fieldNote(array('name' => 'firstname', 'text' => '&nbsp;' . (xtc_not_null(ENTRY_FIRST_NAME_TEXT) ? '<span class="inputRequirement">' . ENTRY_FIRST_NAME_TEXT . '</span>' : '')), $this->coo_address->get_('entry_firstname'));

		if(is_array($this->content_array['form_data']['firstname']) == false)
		{
			$this->content_array['form_data']['firstname'] = array();
		}
		$this->content_array['form_data']['firstname']['name'] = 'firstname';
		$this->content_array['form_data']['firstname']['value'] = $this->coo_address->get_('entry_firstname');
		$this->content_array['form_data']['firstname']['required'] = 0;
		$this->content_array['form_data']['firstname']['required_symbol'] = ENTRY_FIRST_NAME_TEXT;
		if((int)ENTRY_FIRST_NAME_MIN_LENGTH > 0)
		{
			$this->content_array['form_data']['firstname']['required'] = 1;
		}
	}

	protected function add_lastname()
	{
		$this->content_array['INPUT_LASTNAME'] = xtc_draw_input_fieldNote(array('name' => 'lastname', 'text' => '&nbsp;' . (xtc_not_null(ENTRY_LAST_NAME_TEXT) ? '<span class="inputRequirement">' . ENTRY_LAST_NAME_TEXT . '</span>' : '')), $this->coo_address->get_('entry_lastname'));
		
		if(is_array($this->content_array['form_data']['lastname']) == false)
		{
			$this->content_array['form_data']['lastname'] = array();
		}
		$this->content_array['form_data']['lastname']['name'] = 'lastname';
		$this->content_array['form_data']['lastname']['value'] = $this->coo_address->get_('entry_lastname');
		$this->content_array['form_data']['lastname']['required'] = 0;
		$this->content_array['form_data']['lastname']['required_symbol'] = ENTRY_LAST_NAME_TEXT;
		if((int)ENTRY_LAST_NAME_MIN_LENGTH > 0)
		{
			$this->content_array['form_data']['lastname']['required'] = 1;
		}
	}

	protected function add_company()
	{
		if(ACCOUNT_COMPANY == 'true')
		{
			$this->content_array['company'] = '1';
			$this->content_array['INPUT_COMPANY'] = xtc_draw_input_fieldNote(array('name' => 'company', 'text' => '&nbsp;' . (xtc_not_null(ENTRY_COMPANY_TEXT) ? '<span class="inputRequirement">' . ENTRY_COMPANY_TEXT . '</span>' : '')), $this->coo_address->get_('entry_company'));

			if(is_array($this->content_array['form_data']['company']) == false)
			{
				$this->content_array['form_data']['company'] = array();
			}
			$this->content_array['form_data']['company']['name'] = 'company';
			$this->content_array['form_data']['company']['value'] = $this->coo_address->get_('entry_company');
			$this->content_array['form_data']['company']['required'] = 0;
			$this->content_array['form_data']['company']['required_symbol'] = ENTRY_COMPANY_TEXT;
			/*
			  if((int)ENTRY_COMPANY_MIN_LENGTH > 0)
			  {
			  $this->content_array['form_data']['company']['required'] = 1;
			  }
			 */
		}
		else
		{
			$this->content_array['company'] = '0';
		}
	}

	protected function add_street_address()
	{
		$this->content_array['INPUT_STREET'] = xtc_draw_input_fieldNote(array('name' => 'street_address', 'text' => '&nbsp;' . (xtc_not_null(ENTRY_STREET_ADDRESS_TEXT) ? '<span class="inputRequirement">' . ENTRY_STREET_ADDRESS_TEXT . '</span>' : '')), $this->coo_address->get_('entry_street_address'));
		
		if(is_array($this->content_array['form_data']['street_address']) == false)
		{
			$this->content_array['form_data']['street_address'] = array();
		}
		$this->content_array['form_data']['street_address']['name'] = 'street_address';
		$this->content_array['form_data']['street_address']['value'] = $this->coo_address->get_('entry_street_address');
		$this->content_array['form_data']['street_address']['required'] = 0;
		$this->content_array['form_data']['street_address']['required_symbol'] = ENTRY_STREET_ADDRESS_TEXT;
		if((int)ENTRY_STREET_ADDRESS_MIN_LENGTH > 0)
		{
			$this->content_array['form_data']['street_address']['required'] = 1;
		}
	}

	protected function add_suburb()
	{
		if(ACCOUNT_SUBURB == 'true')
		{
			$this->content_array['suburb'] = '1';
			$this->content_array['INPUT_SUBURB'] = xtc_draw_input_fieldNote(array('name' => 'suburb', 'text' => '&nbsp;' . (xtc_not_null(ENTRY_SUBURB_TEXT) ? '<span class="inputRequirement">' . ENTRY_SUBURB_TEXT . '</span>' : '')), $this->coo_address->get_('entry_suburb'));

			if(is_array($this->content_array['form_data']['suburb']) == false)
			{
				$this->content_array['form_data']['suburb'] = array();
			}
			$this->content_array['form_data']['suburb']['name'] = 'suburb';
			$this->content_array['form_data']['suburb']['value'] = $this->coo_address->get_('entry_suburb');
			$this->content_array['form_data']['suburb']['required'] = 0;
			$this->content_array['form_data']['suburb']['required_symbol'] = ENTRY_SUBURB_TEXT;
		}
		else
		{
			$this->content_array['suburb'] = '0';
		}
	}

	protected function add_postcode()
	{
		$this->content_array['INPUT_CODE'] = xtc_draw_input_fieldNote(array('name' => 'postcode', 'text' => '&nbsp;' . (xtc_not_null(ENTRY_POST_CODE_TEXT) ? '<span class="inputRequirement">' . ENTRY_POST_CODE_TEXT . '</span>' : '')), $this->coo_address->get_('entry_postcode'));
		
		if(is_array($this->content_array['form_data']['postcode']) == false)
		{
			$this->content_array['form_data']['postcode'] = array();
		}
		$this->content_array['form_data']['postcode']['name'] = 'postcode';
		$this->content_array['form_data']['postcode']['value'] = $this->coo_address->get_('entry_postcode');
		$this->content_array['form_data']['postcode']['required'] = 0;
		$this->content_array['form_data']['postcode']['required_symbol'] = ENTRY_POST_CODE_TEXT;
		if((int)ENTRY_POSTCODE_MIN_LENGTH > 0)
		{
			$this->content_array['form_data']['postcode']['required'] = 1;
		}
	}

	protected function add_city()
	{
		$this->content_array['INPUT_CITY'] = xtc_draw_input_fieldNote(array('name' => 'city', 'text' => '&nbsp;' . (xtc_not_null(ENTRY_CITY_TEXT) ? '<span class="inputRequirement">' . ENTRY_CITY_TEXT . '</span>' : '')), $this->coo_address->get_('entry_city'));
		
		if(is_array($this->content_array['form_data']['city']) == false)
		{
			$this->content_array['form_data']['city'] = array();
		}
		$this->content_array['form_data']['city']['name'] = 'city';
		$this->content_array['form_data']['city']['value'] = $this->coo_address->get_('entry_city');
		$this->content_array['form_data']['city']['required'] = 0;
		$this->content_array['form_data']['city']['required_symbol'] = ENTRY_CITY_TEXT;
		if((int)ENTRY_CITY_MIN_LENGTH > 0)
		{
			$this->content_array['form_data']['city']['required'] = 1;
		}
	}

	protected function add_state()
	{
		if(ACCOUNT_STATE == 'true')
		{
			$this->content_array['state'] = '1';
			
			if(is_array($this->content_array['form_data']['state']) == false)
			{
				$this->content_array['form_data']['state'] = array();
			}
			$this->content_array['form_data']['state']['required'] = 0;
			$this->content_array['form_data']['state']['required_symbol'] = ENTRY_STATE_TEXT;
			if((int)ENTRY_STATE_MIN_LENGTH > 0)
			{
				$this->content_array['form_data']['state']['required'] = 1;
			}

			if($this->process == true)
			{
				if($this->entry_state_has_zones == true)
				{
					$t_zones_array = array();
					$t_zones_query = xtc_db_query("select zone_name from " . TABLE_ZONES . " where zone_country_id = '" . xtc_db_input($this->customer_country_id) . "' order by zone_name");
					while($t_zones_values = xtc_db_fetch_array($t_zones_query))
					{
						$t_zones_array[] = array('id' => $t_zones_values['zone_name'], 'text' => $t_zones_values['zone_name']);
					}
					$t_state_input = xtc_draw_pull_down_menuNote(array('name' => 'state', 'text' => '&nbsp;' . (xtc_not_null(ENTRY_STATE_TEXT) ? '<span class="inputRequirement">' . ENTRY_STATE_TEXT . '</span>' : '')), $t_zones_array);

					$this->content_array['form_data']['state']['name'] = 'state';
					$this->content_array['form_data']['state']['type'] = 'selection';
					if($this->coo_address->get_('address_book_id') != 0)
					{
						$this->content_array['form_data']['state']['value'] = $this->coo_address->get_('entry_state');
					}
					else
					{
						$this->content_array['form_data']['state']['value'] = $this->customer_state;
					}

					$this->content_array['zones_data'] = $t_zones_array;
				}
				else
				{
					$t_state_input = xtc_draw_input_fieldNote(array('name' => 'state', 'text' => '&nbsp;' . (xtc_not_null(ENTRY_STATE_TEXT) ? '<span class="inputRequirement">' . ENTRY_STATE_TEXT . '</span>' : '')));

					$this->content_array['form_data']['state']['name'] = 'state';
					$this->content_array['form_data']['state']['value'] = xtc_get_zone_name($this->coo_address->get_('entry_country_id'), $this->coo_address->get_('entry_zone_id'), $this->coo_address->get_('entry_state'));
					$this->content_array['form_data']['state']['type'] = 'input';
				}
			}
			else
			{
				$t_state_input = xtc_draw_input_fieldNote(array('name' => 'state', 'text' => '&nbsp;' . (xtc_not_null(ENTRY_STATE_TEXT) ? '<span class="inputRequirement">' . ENTRY_STATE_TEXT . '</span>' : '')), xtc_get_zone_name($this->coo_address->get_('entry_country_id'), $this->coo_address->get_('entry_zone_id'), $this->coo_address->get_('entry_state')));

				$this->content_array['form_data']['state']['name'] = 'state';
				$this->content_array['form_data']['state']['value'] = xtc_get_zone_name($this->coo_address->get_('entry_country_id'), $this->coo_address->get_('entry_zone_id'), $this->coo_address->get_('entry_state'));
				$this->content_array['form_data']['state']['type'] = 'input';
			}

			$this->content_array['INPUT_STATE'] = $t_state_input;
		}
		else
		{
			$this->content_array['state'] = '0';
		}
	}

	protected function add_country()
	{
		if($this->customer_country_id)
		{
			$t_selected = htmlentities_wrapper($this->customer_country_id);
		}
		else
		{
			$t_selected = $this->coo_address->get_('entry_country_id');
		}

		if(is_array($this->content_array['form_data']['country']) == false)
		{
			$this->content_array['form_data']['country'] = array();
		}
		$this->content_array['form_data']['country']['name'] = 'country';
		$this->content_array['form_data']['country']['value'] = $t_selected;
		$this->content_array['form_data']['country']['required'] = 1;
		$this->content_array['form_data']['country']['required_symbol'] = ENTRY_STATE_TEXT;
		$this->content_array['countries_data'] = xtc_get_countriesList();
		$this->content_array['SELECT_COUNTRY'] = xtc_get_country_list(array('name' => 'country', 'text' => '&nbsp;' . (xtc_not_null(ENTRY_COUNTRY_TEXT) ? '<span class="inputRequirement">' . ENTRY_COUNTRY_TEXT . '</span>' : '')), $t_selected);
	}

	protected function add_privacy()
	{
		$this->content_array['GM_PRIVACY_LINK'] = gm_get_privacy_link('GM_CHECK_PRIVACY_ACCOUNT_ADDRESS_BOOK');
	}

	protected function add_primary_address()
	{
		if((isset($this->action_edit) && ($this->customer_default_address_id != $this->coo_address->get_('address_book_id')) || is_null($this->coo_address->get_('address_book_id'))) || (isset($this->action_edit) == false))
		{
			$this->content_array['new'] = '1';
			$this->content_array['CHECKBOX_PRIMARY'] = xtc_draw_checkbox_field('primary', 'on', false, 'id="primary"');
			
			if(is_array($this->content_array['checkbox_primary_data']) == false)
			{
				$this->content_array['checkbox_primary_data'] = array();
			}
			$this->content_array['checkbox_primary_data']['NAME'] = 'primary';
			$this->content_array['checkbox_primary_data']['VALUE'] = 'on';
			$this->content_array['checkbox_primary_data']['ID'] = 'primary';
		}
		else
		{
			$this->content_array['new'] = '0';
		}
	}
}