<?php
/* --------------------------------------------------------------
  FilterBoxContentView.inc.php 2014-09-21 gm
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2014 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------
 */
?><?php
require_once (DIR_FS_INC . 'xtc_get_all_get_params.inc.php');

class FilterBoxContentView extends ContentView
{
	/*
	 * constructor
	 */
	function FilterBoxContentView()
	{
		parent::__construct();
		$this->set_content_template('boxes/box_filter.html');
	}

	/*
	 * create HTML code for every feature
	 * @param int $p_categories_id  category_id
	 * @param int $p_language_id  shop language_id
	 * @param array $p_selected_feature_value_id_array  array with feature value ids
	 * @return array $t_html_output  the HTML code as array for output
	 */
	function get_html($p_categories_id, $p_language_id, $p_selected_feature_value_id_array = false, $p_price_start = false, $p_price_end = false, $p_filter_url = false)
	{
		$c_categories_id = (int)$p_categories_id;
		$c_language_id = (int)$p_language_id;
		$t_deselection_flag = false;

		$coo_feature_set_source = MainFactory::create_object('FeatureSetSource');

		$t_coo_control = MainFactory::create_object('FeatureControl');
		$t_coo_cat_filter = $t_coo_control->get_categories_filter_array(array('categories_id' => $c_categories_id), array('sort_order'));

		$t_feature_mode = gm_get_conf('FEATURE_MODE');
		$t_feature_display_mode = gm_get_conf('FEATURE_DISPLAY_MODE');
		$t_feature_empty_box_mode = gm_get_conf('FEATURE_EMPTY_BOX_MODE');
		if($c_categories_id > 0)
		{
			$coo_categories = MainFactory::create_object('GMDataObject', array('categories', array('categories_id' => $c_categories_id)));
			$t_feature_mode = $coo_categories->get_data_value('feature_mode');
			$t_feature_display_mode = $coo_categories->get_data_value('feature_display_mode');
		}

		$t_selected_feature_value_id_array = array();
		if(is_array($p_selected_feature_value_id_array))
		{
			if(is_object($GLOBALS['coo_debugger']))
			{
				$GLOBALS['coo_debugger']->log('FilterBoxContentView get_html() $p_selected_feature_value_id_array: ' . print_r($p_selected_feature_value_id_array, true), 'FilterManager');
			}
			$t_selected_feature_value_id_array = $p_selected_feature_value_id_array;
			$t_feature_values = $coo_feature_set_source->convert_values_array_to_feature_values_array($p_selected_feature_value_id_array);
		}
		else
		{
			$t_allowed_features = false;
			if($t_feature_mode == 1)
			{
				$t_allowed_features = array();
				$t_features_array = $coo_feature_set_source->extract_features_from_feature_values_string($p_selected_feature_value_id_array);
				foreach($t_coo_cat_filter as $t_coo_filter)
				{
					if(!in_array($t_coo_filter->v_feature_id, $t_features_array))
					{
						break;
					}
					$t_allowed_features[] = $t_coo_filter->v_feature_id;
				}
			}

			$coo_feature_set_source = MainFactory::create_object('FeatureSetSource');
			$t_selected_feature_value_id_array = $coo_feature_set_source->extract_values_from_feature_values_string($p_selected_feature_value_id_array, $t_allowed_features);
			$t_feature_values = $p_selected_feature_value_id_array;
		}

		if($t_feature_values == '')
		{
			$_SESSION["filter_history"] = array();
		}

		$t_html_output = '';
		$t_html_array = array();

		$t_all_feature_values_of_all_sets = $coo_feature_set_source->get_available_feature_values_by_feature_values($c_categories_id, array());
		$t_set_bound_feature_values = $coo_feature_set_source->get_available_feature_values_by_feature_values($c_categories_id, $t_feature_values);
		$t_selected_values_count = 0;
		foreach($t_coo_cat_filter as $t_coo_filter)
		{
			$t_feature_id = $t_coo_filter->v_feature_id;
			$t_template = $t_coo_filter->v_selection_template;
			$t_value_conjunction = $t_coo_filter->v_value_conjunction;
			$t_feature_name = $t_coo_filter->get_feature_name($c_language_id);
			$t_feature_value_array = $t_coo_control->get_feature_value_array($t_feature_id);
			$t_feature_value_array = $t_coo_control->get_feature_value_description($t_feature_value_array);

			$t_feature_empty = true;

			$t_feature_value_data_array = array();
			foreach($t_feature_value_array as $f_coo_feature)
			{
				if(!in_array($f_coo_feature['feature_value_id'], $t_all_feature_values_of_all_sets))
					continue;
				if(!in_array($f_coo_feature['feature_value_id'], $t_set_bound_feature_values) && $t_feature_display_mode == 0)
					continue;
				$t_visible = true;
				if(!in_array($f_coo_feature['feature_value_id'], $t_set_bound_feature_values))
				{
					$t_visible = false;
				}
				else
				{
					$t_feature_empty = false;
				}

				$t_feature_value_id = $f_coo_feature['feature_value_id'];
				$t_feature_value_name = $f_coo_feature['feature_value_text_array'][$c_language_id];

				# feature_value_id selected TRUE/FALSE
				$t_feature_value_selected = in_array($t_feature_value_id, $t_selected_feature_value_id_array);
				if($t_feature_value_selected)
					$t_selected_values_count++;

				$t_feature_value_data_array[] = array(
					'ID' => $t_feature_value_id,
					'NAME' => $t_feature_value_name,
					'SELECTED' => $t_feature_value_selected,
					'DISPLAY_MODE' => $t_feature_display_mode,
					'VISIBLE' => $t_visible
				);
			}

			$t_coo_content_view = MainFactory::create_object('ContentView');
			$t_coo_content_view->set_content_template('module/filter_selection/' . $t_template);


			$t_coo_content_view->set_content_data('FEATURE_NAME', $t_feature_name);
			$t_coo_content_view->set_content_data('FEATURE_VALUE_DATA', $t_feature_value_data_array);
			$t_coo_content_view->set_content_data('FEATURE_ID', $t_feature_id);
			$t_coo_content_view->set_content_data('VALUE_CONJUNCTION', (int)$t_value_conjunction);

			$t_content_array = array();
			$t_content_array['html'] = $t_coo_content_view->get_html(0);
			$t_content_array['empty_feature'] = $t_feature_empty;

			$t_html_array[] = $t_content_array;
		}

		$t_html_array_count = count($t_html_array);
		for($i = 0; $i < $t_html_array_count; $i++)
		{
			if($i == 0)
			{
				$t_html_array[$i]['show'] = true;
			}
			else
			{
				if(strpos($t_html_array[$i - 1]['html'], 'checked') || strpos($t_html_array[$i - 1]['html'], 'selected'))
				{
					$t_html_array[$i]['show'] = true;
				}
				else
				{
					$t_html_array[$i]['show'] = false;
				}
			}
		}
		if($t_selected_values_count == 0)
		{
			$_SESSION["filter_history"] = array();
		}

		# contains html code for feature_value selections
		$this->set_content_data('FEATURE_DATA', $t_html_array);

		$this->set_content_data('FEATURE_MODE', $t_feature_mode);
		$this->set_content_data('FEATURE_DISPLAY_MODE', $t_feature_display_mode);
		$this->set_content_data('FEATURE_EMPTY_BOX_MODE', $t_feature_empty_box_mode);
		$this->set_content_data('categories_id', (int)$c_categories_id);

		$coo_seo_boost = MainFactory::create_object('GMSEOBoost');
		if($coo_seo_boost->boost_categories == true)
		{
			# use boosted url
			$t_action_url = $coo_seo_boost->get_current_boost_url();
		}
		else
		{
			# use default url for splitting urls
			$t_all_get_params = xtc_get_all_get_params();
			$t_action_url = basename($_SERVER['PHP_SELF']) . '?' . trim(urldecode($t_all_get_params), '&');
		}
		if($p_filter_url != false)
		{
			$t_action_url = $p_filter_url;
		}
		$t_action_url = xtc_href_link($t_action_url, '', 'NONSSL', true, true, true);

		//$t_action_url = gm_get_env_info('REQUEST_URI');
		$this->set_content_data('FILTER_URL', $t_action_url);
		$this->set_content_data('FORM_ACTION_URL', $t_action_url);
		$this->set_content_data('CURRENCY', $_SESSION['currency']);

		# entered prices
		$t_price_start = '';
		$t_price_end = '';

		if($p_price_start !== false && !empty($p_price_start))
		{
			$t_price_start = htmlentities_wrapper($p_price_start);
		}
		if($p_price_end !== false && !empty($p_price_end))
		{
			$t_price_end = htmlentities_wrapper($p_price_end);
		}

		$this->set_content_data('DEFAULT_PRICE_START', $t_price_start);
		$this->set_content_data('DEFAULT_PRICE_END', $t_price_end);

		$this->set_content_data('PRICE_FILTER_FROM_ACTIVE', gm_get_conf('PRICE_FILTER_FROM_ACTIVE'));
		$this->set_content_data('PRICE_FILTER_TO_ACTIVE', gm_get_conf('PRICE_FILTER_TO_ACTIVE'));

		$t_html_output = $this->build_html();
		return $t_html_output;
	}
}