<?php
/* --------------------------------------------------------------
  GxmlMaster.inc.php 2014-02-25 gm
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2014 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------
 */

require_once(DIR_FS_CATALOG . 'inc/xtc_get_geo_zone_code.inc.php');

abstract class GxmlMaster {

	protected $v_mapper_array = array();
	protected $v_table_mapper_array = array();
	protected $v_language_dependent_mapper_array = array();
	protected $v_coo_gxml_helper;
	protected $v_coo_response_xml;
	protected $v_plural_name;
	protected $v_singular_name;
	protected $v_language_array;
	protected $v_tax_array;

	abstract protected function add_node($p_data_array, $p_table_column, $p_xml_name, $p_coo_xml);
	abstract protected function delete_object(SimpleXMLElement $p_coo_xml);

	public function __construct()
	{
		$this->v_coo_gxml_helper = MainFactory::create_object('GxmlHelper');
		$this->v_coo_response_xml = simplexml_load_string($this->v_coo_gxml_helper->get_blank_xml());
		$this->load_languages();
		$this->load_taxes();
	}

	protected function set_plural_name($p_name)
	{
		$this->v_plural_name = $p_name;
	}
	
	
	protected function set_singular_name($p_name)
	{
		$this->v_singular_name = $p_name;
	}
	
	
	protected function get_plural_name()
	{
		return $this->v_plural_name;
	}
	
	
	protected function get_singular_name()
	{
		return $this->v_singular_name;
	}
	
	
	public function add($p_data_array, $p_coo_xml, $p_additional_content = array())
	{
		foreach ($this->v_mapper_array AS $t_xml_name => $t_table_column)
		{
			$this->add_node($p_data_array, $t_table_column, $t_xml_name, $p_coo_xml);
		}
	}
	
	
	public function node_exists($p_coo_xml, $p_node_name, $p_attributes_array = array(), $p_attribute_conjunction = 'and')
	{
		$t_node_children = $p_coo_xml->children();
		
		foreach ($t_node_children as $t_node_name => $t_node_values_array)
		{
			if ($t_node_name == $p_node_name)
			{
				if (empty($p_attributes_array))
				{
					return true;
				}
				else
				{
					$t_node_exists = null;
					foreach ($p_attributes_array as $t_attribute_name => $t_attribute_value)
					{
						$t_node_values_attributes = $t_node_values_array->attributes();
						switch ($p_attribute_conjunction)
						{
							case 'or':
								if ($t_node_exists === null)
								{
									$t_node_exists = false;
								}
								
								$t_node_exists |= isset($t_node_values_attributes[$t_attribute_name]) && (string)$t_node_values_attributes[$t_attribute_name][0] == $t_attribute_value;
								break;
							case 'and':
								if ($t_node_exists === null)
								{
									$t_node_exists = true;
								}
								$t_node_exists &= isset($t_node_values_attributes[$t_attribute_name]) && (string)$t_node_values_attributes[$t_attribute_name][0] == $t_attribute_value;
								break;
							default:
								break;
						}
					}
					return (boolean)$t_node_exists;
				}
			}			
		}
		
		return false;
	}
	
	
	public function node_value_exists($p_coo_xml, $p_node_name, $p_node_value_name, $p_node_value, $p_attributes_array = array())
	{
		$t_node_exists = false;
		
		foreach($p_coo_xml->{$p_node_name} AS $t_node_name => $t_node_values_array)
		{
			if($t_node_name == $p_node_name)
			{
				if(empty($p_attributes_array))
				{
					foreach($t_node_values_array->children() AS $t_node_value_name => $t_node_value_array)
					{
						if($t_node_value_name == $p_node_value_name && $p_node_value == (string)$t_node_value_array[0])
						{
							$t_node_exists = $t_node_values_array;
							break;
						}
					}
				}
				else
				{
					foreach($p_attributes_array AS $t_attribute_name => $t_attribute_value)
					{
						foreach($t_node_values_array->attributes() AS $t_node_attribute_name => $t_value_array)
						{
							if($t_attribute_name == $t_node_attribute_name && (string)$t_value_array[0] == $t_attribute_value)
							{
								foreach($t_node_values_array->children() AS $t_node_value_name => $t_node_value_array)
								{
									if($t_node_value_name == $p_node_value_name && $p_node_value == (string)$t_node_value_array[0])
									{
										$t_node_exists = $t_node_values_array;
										break;
									}
								}								
							}
						}
					}
				}
			}			
		}
		
		return $t_node_exists;
	}
	
	
	protected function save(&$p_coo_data_object, $p_data_array, $p_primary_key_name = false, $p_primary_key_value = false, $p_update = false)
	{
		$t_success = true;
		$t_new_id = false;
		
		$t_data_array = $this->convert_to_latin1($p_data_array);
		
		if (is_array(current($t_data_array)) && $p_coo_data_object->v_db_table != TABLE_PRODUCTS_XSELL && $p_coo_data_object->v_db_table != 'products_properties_combis_values')
		{
			if(strpos($p_coo_data_object->v_db_table, TABLE_PERSONAL_OFFERS_BY) !== false)
			{
				foreach ($t_data_array AS $p_personal_offer_data_array)
				{
					if($p_update === true)
					{
						$p_coo_data_object->set_keys(array($p_primary_key_name => $p_primary_key_value, 'quantity' => $p_personal_offer_data_array['quantity']));
					}
					else
					{
						$p_coo_data_object->set_keys(array($p_primary_key_name => false));
					}
					
					$p_coo_data_object->set_data_value($p_primary_key_name, $p_primary_key_value);
					
					foreach ($p_personal_offer_data_array as $t_key => $t_value)
					{
						$p_coo_data_object->set_data_value($t_key, $t_value);
					}

					$t_new_id = $this->perform_db_action($p_coo_data_object, 'update');
					if($p_update === true && $t_new_id === false)
					{
						$p_coo_data_object->set_keys(array($p_primary_key_name => false));
						$t_new_id = $this->perform_db_action($p_coo_data_object, 'update');
					}					
					
					$t_success &= (bool) $t_new_id;
				}		
			}
			else
			{
				foreach ($t_data_array as $t_language_id => $p_language_data_array)
				{
					if ($p_update)
					{
						$p_coo_data_object->set_keys(array($p_primary_key_name => $p_primary_key_value, 'language_id' => $t_language_id));
					}
					else
					{
						$p_coo_data_object->set_keys(array($p_primary_key_name => false));
						$p_coo_data_object->set_data_value('language_id', $t_language_id);
					}

					foreach ($p_language_data_array as $t_key => $t_value)
					{
						$p_coo_data_object->set_data_value($t_key, $t_value);
					}

					if($p_primary_key_name !== false && $p_primary_key_value !== false)
					{
						if($p_primary_key_name != 'language_id' || $p_primary_key_name == 'language_id' && in_array($p_coo_data_object->v_db_table, $this->v_language_dependent_mapper_array))
						{
							$p_coo_data_object->set_data_value($p_primary_key_name, $p_primary_key_value);
						}
					}
					
					$t_new_id = $this->perform_db_action($p_coo_data_object, 'update');
					if($p_update === true && $t_new_id === false)
					{
						$p_coo_data_object->set_keys(array($p_primary_key_name => false));
						$p_coo_data_object->set_data_value('language_id', $t_language_id);
						$t_new_id = $this->perform_db_action($p_coo_data_object, 'update');
					}	
					
					$t_success &= (boolean) $t_new_id;
				}
			}
		}
		elseif ($p_coo_data_object->v_db_table == TABLE_PRODUCTS_XSELL)
		{
			// insert and update
			if(isset($p_data_array['xsell_updates']) && is_array($p_data_array['xsell_updates']))
			{
				foreach($p_data_array['xsell_updates'] as $t_xsell_update)
				{
					$p_coo_data_object->set_data_value('products_id', $p_primary_key_value);
					$p_coo_data_object->set_data_value('products_xsell_grp_name_id', 0);
					$p_coo_data_object->set_data_value('xsell_id', (int)$t_xsell_update['product_id']);
					isset($t_xsell_update['sort_order']) ? $t_sort_order = (int)$t_xsell_update['sort_order'] : $t_sort_order = 0;
					$p_coo_data_object->set_data_value('sort_order', $t_sort_order);
					$this->perform_db_action($p_coo_data_object, 'update', true);
				}
			}
			
			// delete
			if(isset($p_data_array['xsell_deletion']) && is_array($p_data_array['xsell_deletion']))
			{
				foreach($p_data_array['xsell_deletion'] as $t_xsell_deletion)
				{
					$p_coo_data_object->set_keys(array($p_primary_key_name => $p_primary_key_value, 'xsell_id' => $t_xsell_deletion['product_id']));
					$this->perform_db_action($p_coo_data_object, 'delete');
				}
			}
			
			return true;
		}
		else if ($p_coo_data_object->v_db_table == TABLE_PRODUCTS_TO_CATEGORIES)
		{
			$t_categories = explode(',', $t_data_array['categories_id']);
			foreach ($t_categories as $t_category_id)
			{
				$p_coo_data_object->set_data_value($p_primary_key_name, $p_primary_key_value);
				$p_coo_data_object->set_data_value('categories_id', $t_category_id);
				$this->perform_db_action($p_coo_data_object, 'update', true);
			}
			
			$t_delete_categories = explode(',', $t_data_array['delete_categories_id']);
			foreach ($t_delete_categories as $t_category_id)
			{
				if($t_category_id === '')
				{
					continue;
				}
				$p_coo_data_object->set_keys(array($p_primary_key_name => $p_primary_key_value, 'categories_id' => $t_category_id));
				$this->perform_db_action($p_coo_data_object, 'delete');
			}
		}
		else if ($p_coo_data_object->v_db_table == 'products_properties_combis_values')
		{
			foreach ($t_data_array as $t_property_value)
			{
				foreach ($t_property_value as $t_key => $t_value)
				{
					$p_coo_data_object->set_data_value($t_key, $t_value);
				}
				$t_new_id = $this->perform_db_action($p_coo_data_object, 'update', true);
				$t_success &= (boolean) $t_new_id;
			}
		}
		else
		{
			if ($p_update && $p_coo_data_object->v_db_table != TABLE_ORDERS_STATUS_HISTORY)
			{
				$p_coo_data_object->set_keys(array($p_primary_key_name => $p_primary_key_value));
			}
			else
			{
				$p_coo_data_object->set_keys(array($p_primary_key_name => false));
			}
			foreach($t_data_array as $t_key => $t_value)
			{
				$p_coo_data_object->set_data_value($t_key, $t_value);
				
			}
			
			if($p_primary_key_name !== false && $p_primary_key_value !== false)
			{
				$p_coo_data_object->set_data_value($p_primary_key_name, $p_primary_key_value);
			}
			
			$t_new_id = $this->perform_db_action($p_coo_data_object, 'update');
			$t_success &= (boolean) $t_new_id;
		}
		
		return $t_new_id;
	}
	
	
	protected function save_object(SimpleXMLElement $p_coo_xml)
	{
		$t_data_array = $this->build_data_array($p_coo_xml);
		$t_table_name_array = array_keys($t_data_array);
		
		$coo_data_objects = array();
		foreach ($t_table_name_array as $t_table_name)
		{
			$coo_data_objects[$t_table_name] = MainFactory::create_object('GMDataObject', array($t_table_name));
		}
		
		$t_object_id_name = $p_coo_xml->getName() . '_id';
		$t_object_id = (string)$p_coo_xml->{$t_object_id_name};
		$t_update = false;
		if (!empty($t_object_id))
		{
			if (empty($this->v_default_action) || $this->v_default_action == 'update')
			{
				$t_update = true;
			}
		}
		else
		{
			$t_object_id = false;
		}
		
		// save main table
		$t_main_id_column = $this->v_mapper_array[$t_object_id_name];
		$t_main_table_name = $this->v_table_mapper_array[$t_object_id_name];
		$coo_actual_table_object = $coo_data_objects[$t_main_table_name];
		$t_actual_data_array = $t_data_array[$t_main_table_name];
		
		
		$t_new_id = $this->save($coo_actual_table_object, $t_actual_data_array, $t_main_id_column, $t_object_id, $t_update);
		if ($t_update == false)
		{
			$p_coo_xml->addChild($t_object_id_name, $t_new_id);
			$t_data_array = $this->build_data_array($p_coo_xml);
		}
		
		if ($t_new_id == -1 || $t_new_id === true)
		{
			$t_new_id = $t_object_id;
		}
		
		$t_table_name_array = array_flip($t_table_name_array);
		unset($t_table_name_array[$t_main_table_name]);
		$t_table_name_array = array_flip($t_table_name_array);
		
		// save sub tables
		foreach ($t_table_name_array as $t_table_name)
		{
			$coo_actual_table_object = $coo_data_objects[$t_table_name];
			$t_actual_data_array = $t_data_array[$t_table_name];

			$t_id = $this->save($coo_actual_table_object, $t_actual_data_array, $t_main_id_column, $t_new_id, $t_update);
			
			if ($t_update == true && $t_id == false)
			{
				$this->save($coo_actual_table_object, $t_actual_data_array, $t_main_id_column, $t_new_id, false);
			}
		}
		return $t_new_id;
	}
	

	protected function build_where_part($p_data_array, $p_where_block_exists = false)
	{
		$t_where_part = '';
		$t_conditions_group_array = array();
		
		if(is_array($p_data_array) && isset($p_data_array['where_parts']))
		{
			$t_where_part = ' WHERE ((';
			
			if($p_where_block_exists)
			{
				$t_where_part = ' AND ((';
			}			
			
			foreach($p_data_array['where_parts'] AS $t_data_array)
			{
				$t_conditions_array = array();
				
				foreach($t_data_array AS $t_column => $t_value)
				{
					$t_conditions_array[] = $t_column . " = '" . xtc_db_input($t_value) . "'";
				}
				
				$t_conditions_group_array[] = implode(' AND ', $t_conditions_array);
			}
			
			$t_where_part .= implode(') OR (', $t_conditions_group_array);
			$t_where_part .= ')) ';
		}
		
		return $t_where_part;
	}
	
	protected function build_limit_part($p_data_array)
	{
		$t_limit_part = '';
		
		if(is_array($p_data_array) && isset($p_data_array['limit']))
		{
			$c_limit = (int)$p_data_array['limit'];
			$c_offset = 0;
			if(isset($p_data_array['offset']))
			{
				$c_offset = (int)$p_data_array['offset'];
			}
		
			if($c_limit > 0)
			{
				$t_limit_part = ' LIMIT ' . $c_offset . ', ' . $c_limit;
			}
		}
		
		return $t_limit_part;
	}
	
	protected function build_data_array(SimpleXMLElement $p_coo_xml, $p_parent_table = '')
	{
		$t_data_array = array();
		
		foreach($p_coo_xml->children() as $t_node_name => $t_node_values_array)
		{
			$t_node_name = $p_parent_table . $t_node_name;
			if(isset($this->v_table_mapper_array[$t_node_name]) == false || isset($this->v_mapper_array[$t_node_name]) == false)
			{
				if(in_array($t_node_name, array('special', 'categories', 'cross_selling_products')) == false)
				{
					continue;
				}
			}
			
			if(isset($this->v_table_mapper_array[$t_node_name]))
			{
				$t_table = $this->v_table_mapper_array[$t_node_name];
			}
			
			if(isset($this->v_mapper_array[$t_node_name]))
			{
				$t_column = $this->v_mapper_array[$t_node_name];
			}

			switch ($t_node_name)
			{
				case 'special':
				case 'status_history':
					$t_data_array = array_merge($t_data_array, $this->build_data_array($p_coo_xml->{$t_node_name}, $t_node_name . '/'));
					break;
				case 'customer_group_permission':
					$t_data_array[$t_table][$t_column . $t_node_values_array['customer_group_id']] = (string)$t_node_values_array[0];
					break;
				case 'quantity_unit_id':
					$t_data_array[$t_table]['products_id'] = true;
					$t_data_array[$t_table][$this->v_mapper_array[$t_node_name]] = (string)$t_node_values_array[0];
					break;
				case 'cross_selling_products':
					$t_update_xselling_products = array();
					$t_delete_xselling_products = array();
					
					foreach ($t_node_values_array->children() as $t_xsell_product_node)
					{
						$t_xsell_update = array();
						$t_xsell_delete = array();
						
						foreach ($t_xsell_product_node as $t_leaf_name => $t_leaf_value)
						{
							if (isset($t_xsell_product_node['action']) && $t_xsell_product_node['action'] == 'delete')
							{
								$t_xsell_delete[$t_leaf_name] = (string)$t_leaf_value;
								continue;
							}
							else
							{
								$t_xsell_update[$t_leaf_name] = (string)$t_leaf_value;
							}
						}
						if(count($t_xsell_update))
							$t_update_xselling_products[] = $t_xsell_update;
						if(count($t_xsell_delete))
							$t_delete_xselling_products[] = $t_xsell_delete;
					}
					$t_data_array[$this->v_table_mapper_array['cross_selling_products/cross_selling_product/product_id']]['xsell_updates'] = $t_update_xselling_products;
					$t_data_array[$this->v_table_mapper_array['cross_selling_products/cross_selling_product/product_id']]['xsell_deletion'] = $t_delete_xselling_products;
					break;
				case 'categories':
					$t_categories = array();
					$t_delete_categories = array();
					foreach ($t_node_values_array->children() as $t_category_node)
					{
						foreach ($t_category_node->children() as $t_leaf_name => $t_leaf_value)
						{
							if ($t_leaf_name == 'category_id')
							{
								if (isset($t_category_node['action']) && $t_category_node['action'] == 'delete')
								{
									$t_delete_categories[] = $t_leaf_value;
								}
								else
								{
									$t_categories[] = $t_leaf_value;
								}
							}
						}
					}
					if (!empty($t_categories))
					{
						$t_data_array[$this->v_table_mapper_array['categories/category/category_id']]['categories_id'] = implode(',', $t_categories);
						$t_data_array[$this->v_table_mapper_array['categories/category/category_id']]['delete_categories_id'] = implode(',', $t_delete_categories);
					}
					break;
				case 'personal_offer':
					foreach($t_node_values_array->children() as $t_personal_offer_node_name => $t_personal_offer_node_values_array)
					{
						$count_index = 0;
					
						switch($t_personal_offer_node_name)
						{
							case 'price':
								
								if(isset($t_data_array[TABLE_PERSONAL_OFFERS_BY . $t_node_values_array['customer_group_id']]))
								{
									$count_index = count($t_data_array[TABLE_PERSONAL_OFFERS_BY . $t_node_values_array['customer_group_id']]) - 1;
									if(isset($t_data_array[TABLE_PERSONAL_OFFERS_BY . $t_node_values_array['customer_group_id']][$count_index]['price']))
									{
										$count_index++;
									}
								}
								
								$t_data_array[TABLE_PERSONAL_OFFERS_BY . $t_node_values_array['customer_group_id']][$count_index]['personal_offer'] = (string)$t_personal_offer_node_values_array[0];
								break;
							case 'quantity':
								
								if(isset($t_data_array[TABLE_PERSONAL_OFFERS_BY . $t_node_values_array['customer_group_id']]))
								{
									$count_index = count($t_data_array[TABLE_PERSONAL_OFFERS_BY . $t_node_values_array['customer_group_id']]) - 1;
									if(isset($t_data_array[TABLE_PERSONAL_OFFERS_BY . $t_node_values_array['customer_group_id']][$count_index]['quantity']))
									{
										$count_index++;
									}
								}
								
								$t_data_array[TABLE_PERSONAL_OFFERS_BY . $t_node_values_array['customer_group_id']][$count_index]['quantity'] = (string)$t_personal_offer_node_values_array[0];
								break;
						}
						
					}
					
					break;
				case 'personal_offers':
					$t_data_array = array_merge($t_data_array, $this->build_data_array($p_coo_xml->{$t_node_name}, ''));
					break;
				case 'property_values':
					if (!isset($t_data_array['products_properties_combis_values']) || !is_array($t_data_array['products_properties_combis_values']))
					{
						$t_data_array['products_properties_combis_values'] = array();
					}
					
					for ($i = 0; $i < count($t_node_values_array); $i++)
					{
						$t_data_array['products_properties_combis_values'][] = array('properties_values_id' => (int) $t_node_values_array->property_value[$i]->property_value_id,
																					'products_properties_combis_id' => (int) $p_coo_xml->product_combi_id);
					}
					break;
				default:
					if (isset($this->v_mapper_array[$t_node_name]))
					{
						if ((isset($t_node_values_array['language_id']) || isset($t_node_values_array['language_iso'])) && in_array($t_table, $this->v_language_dependent_mapper_array))
						{
							if (!isset($t_node_values_array['language_id']))
							{
								$t_language_id = $this->resolve_language_iso($t_node_values_array['language_iso']);
							}
							else
							{
								$t_language_id = $t_node_values_array['language_id'];
							}
							
							$t_data_array[$t_table][(string)$t_language_id][$t_column] = (string)$t_node_values_array[0];
						}
						else
						{
							$t_data_array[$t_table][$this->v_mapper_array[$t_node_name]] = (string)$t_node_values_array[0];
						}
					}
					break;
			}
		}
		
		return $t_data_array;
	}
	
	
	protected function upload($p_simple_xml, &$t_response_array)
	{
		$t_prefix = $this->v_singular_name;
		
		$t_external_id = (string)$p_simple_xml->{'external_' . $t_prefix . '_id'};
		$t_object_id = (string)$p_simple_xml->{$t_prefix . '_id'};

		$t_delete = (string)$p_simple_xml['action'] == 'delete';
		
		if (empty($t_external_id) && empty($t_object_id) && $t_delete === false)
		{
			$t_response_array[] = array(
									'external_' . $t_prefix . '_id' => '',
									$t_prefix . '_id' => $t_object_id,
									'success' => '0',
									'errormessage' => 'missing external_' . $t_prefix . '_id',
									'action_performed' => 'error'
								);
		}
		else if ($t_delete)
		{
			$t_response_array[] = array(
								'external_' . $t_prefix . '_id' => $t_external_id,
								$t_prefix . '_id' => $t_object_id,
								'errormessage' => '',
								'action_performed' => 'delete'
							);
			$t_success = $this->delete_object($p_simple_xml);
			$t_response_array[count($t_response_array) - 1]['success'] = (int)$t_success;
		}
		else
		{
			$t_response_array[] = array(
								'external_' . $t_prefix . '_id' => $t_external_id,
								'errormessage' => '',
								'action_performed' => (empty($t_object_id) ? 'create' : 'update')
							);
			$t_object_id = $this->save_object($p_simple_xml);
			
			$t_success = (boolean) $t_object_id;
			$t_response_array[count($t_response_array) - 1][$t_prefix . '_id'] = $t_object_id;
			$t_response_array[count($t_response_array) - 1]['success'] = (int)$t_success;
		}
		
		return $t_object_id;
	}
	
	protected function upload_children($p_simple_xml, $p_delete)
	{
		if ($p_delete)
		{
			$t_success = $this->delete_children($p_simple_xml);
		}
		else
		{
			$t_success = $this->save_children($p_simple_xml);
		}
		
		return $t_success;
	}
	
	protected function perform_db_action(&$p_sql_object, $p_action = false, $p_replace = false)
	{
		$t_success = true;
		
		if (is_string($p_sql_object))
		{
			$t_success = xtc_db_query($p_sql_object);
		}
		else
		{
			switch ($p_action)
			{
				case 'create':
				case 'update':
					$t_success = $p_sql_object->save_body_data($p_replace);
					$t_success = $t_success === 0 ? -1 : $t_success;
					break;
				case 'delete':
					$t_success = $p_sql_object->delete();
					break;
				default:
					$t_success = false;
					break;
			}
		}
		
		return $t_success;
	}
	
	
	private function add_shipping_times()
	{
		$coo_shipping_times = $this->v_coo_response_xml->addChild('shipping_times');
		$t_sql = "SELECT 
						shipping_status_id,
						language_id,
						shipping_status_name,
						number_of_days,
						shipping_quantity
					FROM " . TABLE_SHIPPING_STATUS . "
					ORDER BY
						shipping_status_id,
						language_id";
		$t_result = xtc_db_query($t_sql);
		$t_shipping_status_id = 0;
		while($t_result_array = xtc_db_fetch_array($t_result))
		{
			if($t_result_array['shipping_status_id'] != $t_shipping_status_id)
			{
				$coo_shipping_time = $coo_shipping_times->addChild('shipping_time');
				$coo_shipping_time->addChild('shipping_time_id', $t_result_array['shipping_status_id']);
				$coo_shipping_time->addChild('number_of_days', $t_result_array['number_of_days']);
				$coo_shipping_time->addChild('quantity', $t_result_array['shipping_quantity']);
			}
						
			$coo_name = $coo_shipping_time->addChild('name', $this->wrap_value($t_result_array['shipping_status_name']));
			$coo_name->addAttribute('language_id', $t_result_array['language_id']);
			$coo_name->addAttribute('language_iso', $this->v_language_array[$t_result_array['language_id']]['iso']);
						
			$t_shipping_status_id = $t_result_array['shipping_status_id'];
		}
	}
	
	
	private function add_tax_classes()
	{
		$this->load_taxes();
		
		$coo_tax_classes = $this->v_coo_response_xml->addChild('tax_classes');
		foreach($this->v_tax_array as $t_tax_class)
		{
			$coo_tax_class = $coo_tax_classes->addChild('tax_class');
			$coo_tax_class->addChild('tax_class_id', $t_tax_class['tax_class_id']);
			$coo_tax_class->addChild('title', $this->wrap_value($t_tax_class['title']));
			$coo_description = $coo_tax_class->addChild('description', $this->wrap_value($t_tax_class['description']));
			$coo_description->addAttribute('type', 2);
			$coo_tax_class->addChild('tax_rate', $t_tax_class['tax_rate']);
			$coo_tax_class->addChild('date_added', $t_tax_class['date_added']);
			$coo_tax_class->addChild('last_modified', $t_tax_class['last_modified']);
		}
		
		return $coo_tax_classes;
	}
		
	
	private function add_quantity_units()
	{
		$coo_quantity_units = $this->v_coo_response_xml->addChild('quantity_units');
		$t_sql = "SELECT 
						quantity_unit_id,
						language_id,
						unit_name
					FROM quantity_unit_description
					ORDER BY
						quantity_unit_id,
						language_id";
		$t_result = xtc_db_query($t_sql);
		$t_quantity_unit_id = 0;
		while($t_result_array = xtc_db_fetch_array($t_result))
		{
			if($t_result_array['quantity_unit_id'] != $t_quantity_unit_id)
			{
				$coo_quantity_unit = $coo_quantity_units->addChild('quantity_unit');
				$coo_quantity_unit->addChild('quantity_unit_id', $t_result_array['quantity_unit_id']);
			}
						
			$coo_quantity_unit_name = $coo_quantity_unit->addChild('name', $this->wrap_value($t_result_array['unit_name']));
			$coo_quantity_unit_name->addAttribute('language_id', $t_result_array['language_id']);
			$coo_quantity_unit_name->addAttribute('language_iso', $this->v_language_array[$t_result_array['language_id']]['iso']);
						
			$t_quantity_unit_id = $t_result_array['quantity_unit_id'];
		}
	}
	
	
	private function add_base_price_units()
	{
		$coo_base_price_units = $this->v_coo_response_xml->addChild('base_price_units');
		$t_sql = "SELECT 
						products_vpe_id,
						language_id,
						products_vpe_name
					FROM " . TABLE_PRODUCTS_VPE . "
					ORDER BY
						products_vpe_id,
						language_id";
		$t_result = xtc_db_query($t_sql);
		$t_products_vpe_id = 0;
		while($t_result_array = xtc_db_fetch_array($t_result))
		{
			if($t_result_array['products_vpe_id'] != $t_products_vpe_id)
			{
				$coo_base_price_unit = $coo_base_price_units->addChild('base_price_unit');
				$coo_base_price_unit->addChild('base_price_unit_id', $t_result_array['products_vpe_id']);
			}
						
			$coo_name = $coo_base_price_unit->addChild('name', $this->wrap_value($t_result_array['products_vpe_name']));
			$coo_name->addAttribute('language_id', $t_result_array['language_id']);
			$coo_name->addAttribute('language_iso', $this->v_language_array[$t_result_array['language_id']]['iso']);
						
			$t_products_vpe_id = $t_result_array['products_vpe_id'];
		}
	}
	
	
	private function add_customer_groups()
	{
		$coo_customer_groups = $this->v_coo_response_xml->addChild('customer_groups');
		$t_sql = "SELECT 
						customers_status_id,
						language_id,
						customers_status_name,
						customers_status_show_price_tax
					FROM " . TABLE_CUSTOMERS_STATUS . "
					ORDER BY
						customers_status_id,
						language_id";
		$t_result = xtc_db_query($t_sql);
		$t_customers_status_id = '';
		while($t_result_array = xtc_db_fetch_array($t_result))
		{
			if($t_result_array['customers_status_id'] != $t_customers_status_id)
			{
				$coo_customer_group = $coo_customer_groups->addChild('customer_group');
				$coo_customer_group->addChild('customer_status_id', $t_result_array['customers_status_id']);
				$coo_customer_group->addChild('include_taxes', $t_result_array['customers_status_show_price_tax']);
			}
						
			$coo_name = $coo_customer_group->addChild('name', $this->wrap_value($t_result_array['customers_status_name']));
			$coo_name->addAttribute('language_id', $t_result_array['language_id']);
			$coo_name->addAttribute('language_iso', $this->v_language_array[$t_result_array['language_id']]['iso']);
						
			$t_customers_status_id = $t_result_array['customers_status_id'];
		}
	}
	
	
	protected function load_languages()
	{
		if (empty($this->v_language_array))
		{
			$this->v_language_array = array();
			
			$t_sql = "SELECT 
							languages_id,
							name,
							code,
							language_charset
						FROM " . TABLE_LANGUAGES. "
						ORDER BY
							languages_id";
			$t_result = xtc_db_query($t_sql);
			
			while($t_result_array = xtc_db_fetch_array($t_result))
			{
				$this->v_language_array[$t_result_array['languages_id']] = array();
				$this->v_language_array[$t_result_array['languages_id']]['language_name'] = $t_result_array['name'];
				$this->v_language_array[$t_result_array['languages_id']]['iso'] = $t_result_array['code'];
				$this->v_language_array[$t_result_array['languages_id']]['charset'] = $t_result_array['language_charset'];
			}
		}
	}
	
	
	protected function load_taxes()
	{
		if (empty($this->v_tax_array))
		{
			$this->v_tax_array = array();
			
			$t_sql = "SELECT 
						tax_class.tax_class_id,
						tax_class.tax_class_title,
						tax_rates.tax_rate,
						tax_class.tax_class_description,
						tax_class.last_modified,
						tax_class.date_added
					FROM " . TABLE_TAX_CLASS . " tax_class ," . TABLE_TAX_RATES . "  tax_rates
					WHERE 
						tax_class.tax_class_id = tax_rates.tax_class_id AND
						tax_rates.tax_zone_id = " . (int)xtc_get_geo_zone_code(STORE_COUNTRY) . "
					ORDER BY
						tax_class.tax_class_id";
			$t_result = xtc_db_query($t_sql);
			
			while($t_result_array = xtc_db_fetch_array($t_result))
			{
				$this->v_tax_array[$t_result_array['tax_class_id']] = array();
				$this->v_tax_array[$t_result_array['tax_class_id']]['tax_class_id'] = $t_result_array['tax_class_id'];
				$this->v_tax_array[$t_result_array['tax_class_id']]['title'] = $t_result_array['tax_class_title'];
				$this->v_tax_array[$t_result_array['tax_class_id']]['description'] = $t_result_array['tax_class_description'];
				$this->v_tax_array[$t_result_array['tax_class_id']]['tax_rate'] = $t_result_array['tax_rate'];
				$this->v_tax_array[$t_result_array['tax_class_id']]['date_added'] = $t_result_array['date_added'];
				$this->v_tax_array[$t_result_array['tax_class_id']]['last_modified'] = $t_result_array['last_modified'];
			}
		}
	}
	
	
	protected function resolve_language_iso($p_language_iso)
	{
		foreach ($this->v_language_array as $t_language_id => $t_language_data)
		{
			if ($t_language_data['iso'] == $p_language_iso)
			{
				return $t_language_id;
			}
		}
		return false;
	}
	
	
	private function add_languages()
	{
		$coo_languages = $this->v_coo_response_xml->addChild('languages');
		
		foreach ($this->v_language_array as $t_language_id => $t_language_data)
		{
			$coo_language = $coo_languages->addChild('language');
			
			$coo_language->addChild('language_id', $t_language_id);
			$coo_language->addChild('name', $this->wrap_value($t_language_data['language_name']));
			$coo_language->addChild('iso', $t_language_data['iso']);
			$coo_language->addChild('charset', $t_language_data['charset']);
		}
	}
	
	
	private function add_order_status()
	{
		$coo_order_statuses = $this->v_coo_response_xml->addChild('order_statuses');
		
		$t_sql = "SELECT 
						orders_status_id,
						language_id,
						orders_status_name
					FROM " . TABLE_ORDERS_STATUS. "
					ORDER BY
						orders_status_id, language_id";
		$t_result = xtc_db_query($t_sql);
		$t_order_status_id = '';
		while($t_result_array = xtc_db_fetch_array($t_result))
		{
			if($t_result_array['orders_status_id'] != $t_order_status_id)
			{
				$coo_order_status = $coo_order_statuses->addChild('order_status');
				$coo_order_status->addChild('orders_status_id', $t_result_array['orders_status_id']);
			}
			$coo_name = $coo_order_status->addChild('name', $this->wrap_value($t_result_array['orders_status_name']));
			$coo_name->addAttribute('language_id', $t_result_array['language_id']);
			$coo_name->addAttribute('language_iso', $this->v_language_array[$t_result_array['language_id']]['iso']);
			
			$t_order_status_id = $t_result_array['orders_status_id'];
		}
	}


	protected function add_blocks($p_blocks_array)
	{
		foreach($p_blocks_array AS $t_type)
		{
			switch($t_type)
			{
				case 'languages':
					$this->add_languages();
					break;
				case 'customer_groups':
					$this->add_customer_groups();
					break;
				case 'shipping_times':
					$this->add_shipping_times();
					break;
				case 'tax_classes':
					$this->add_tax_classes();
					break;
				case 'base_price_units':
					$this->add_base_price_units();
					break;
				case 'quantity_units':
					$this->add_quantity_units();
					break;
				case 'order_status':
					$this->add_order_status();
					break;
			}
		}
	}
	
	
	protected function build_sql_strings($p_coo_paramters_xml, $p_types_array)
	{
		$t_sql_strings_array = array();
		
		foreach($p_types_array AS $t_type)
		{
			switch($t_type)
			{
				case 'limit':
					
					if(isset($p_coo_paramters_xml->limit) && (int)$p_coo_paramters_xml->limit > 0)
					{
						$t_sql_strings_array['limit'] = (int)$p_coo_paramters_xml->limit;

						if(isset($p_coo_paramters_xml->offset) && (int)$p_coo_paramters_xml->offset)
						{
							$t_sql_strings_array['offset'] = (int)$p_coo_paramters_xml->offset;
						}
					}
					
					break;
				case 'where':
					if(isset($p_coo_paramters_xml->{$this->get_plural_name()}) && isset($p_coo_paramters_xml->{$this->get_plural_name()}->{$this->get_singular_name()}))
					{
						$coo_singular_nodes = $p_coo_paramters_xml->{$this->get_plural_name()}->{$this->get_singular_name()};

						for($i = 0; $i < count($coo_singular_nodes); $i++)
						{
							$t_mapper_keys_array = $this->build_path_array($coo_singular_nodes[$i]);

							for($j = 0; $j < count($t_mapper_keys_array); $j++)
							{
								if(isset($this->v_mapper_array[$t_mapper_keys_array[$j]]) && $this->v_mapper_array[$t_mapper_keys_array[$j]] != '')
								{
									$coo_node = $coo_singular_nodes[$i]->xpath($t_mapper_keys_array[$j]);
									$t_sql_strings_array['where_parts'][$i][$this->v_table_mapper_array[$t_mapper_keys_array[$j]] . '.' . $this->v_mapper_array[$t_mapper_keys_array[$j]]] = (string)$coo_node[0];
								}	
							}				
						}
					}
					
					break;
			}
		}
		
		return $t_sql_strings_array;
	}
	
	
	protected function build_path_array($p_coo_xml, $p_path = '')
	{
		$t_path_array = array();
		
		foreach($p_coo_xml->children() AS $t_child_name => $coo_child)
		{
			$t_sub_child_count = count($coo_child->children());
			if($t_sub_child_count > 0)
			{
				$t_path_array = array_merge($t_path_array, $this->build_path_array($coo_child, $p_path . $t_child_name . '/'));
			}
			else
			{
				$t_path_array[] = $p_path . $t_child_name;
			}
		}
		
		return $t_path_array;
	}
	
	
	function build_response_xml($p_response_array)
	{
		$t_response_xml_content = $this->v_coo_gxml_helper->get_blank_xml();
		$coo_response_xml = simplexml_load_string($t_response_xml_content); 
		$t_total_success = true;
		
		$coo_response_xml->addChild('request');
		$coo_response_xml->addChild($this->v_plural_name);
		
		foreach($p_response_array as $t_response_item)
		{
			$t_child = $coo_response_xml->{$this->v_plural_name}->addChild($this->v_singular_name);
			if(isset($t_response_item['external_' . $this->v_singular_name . '_id']))
			{
				$t_child->addChild('external_' . $this->v_singular_name . '_id', $t_response_item['external_' . $this->v_singular_name . '_id']);
			}
			$t_child->addChild($this->v_singular_name . '_id', $t_response_item[$this->v_singular_name . '_id']);
			$t_child->addChild('success', $t_response_item['success']);
			$t_child->addChild('errormessage', $t_response_item['errormessage']);
			$t_child->addChild('action_performed', $t_response_item['action_performed']);
			$t_total_success &= (boolean) $t_response_item['success'];
		}
		
		if (!$t_total_success)
		{
			$coo_response_xml->request->addChild('errormessage', 'request error');
		}
		$coo_response_xml->request->addChild('success', $t_total_success);
		
		return $coo_response_xml;
	}
	
	
	public function convert_to_latin1($p_data)
	{
		if(is_array($p_data))
		{
			foreach($p_data AS $t_key => $t_value)
			{
				$t_key = $this->utf8_decode($t_key);
				
				$t_converted_data[$t_key] = $this->convert_to_latin1($t_value);
			}
		}
		elseif(is_object($p_data))
		{
			$t_vars = array_keys(get_object_vars($p_data));
			$t_converted_data = $p_data;
			
			foreach($t_vars AS $t_var)
			{
				$t_converted_data->$t_var = $this->convert_to_latin1($p_data->$t_var);
			}
		}
		elseif(is_string($p_data))
		{
			$t_converted_data = $this->utf8_decode($p_data);
		}
		else
		{
			$t_converted_data = $p_data;
		}
		
		return $t_converted_data;
	}
	
	
	public function is_utf8_string($p_string)
	{
		if(preg_match('/(?:[\xC2-\xDF][\x80-\xBF]|\xE0[\xA0-\xBF][\x80-\xBF]|[\xE1-\xEC\xEE\xEF][\x80-\xBF]{2}|\xED[\x80-\x9F][\x80-\xBF]|\xF0[\x90-\xBF][\x80-\xBF]{2}|[\xF1-\xF3][\x80-\xBF]{3}|\xF4[\x80-\x8F][\x80-\xBF]{2})+/xs', $p_string))
		{
			return true;
		}
		
		return false;
	}
	
	
	public function utf8_decode($p_string)
	{
		if($this->is_utf8_string($p_string))
		{
			$t_target_charset = 'ISO-8859-15';
		
			if(isset($_SESSION['language_charset']))
			{
				$t_target_charset = trim(strtoupper($_SESSION['language_charset']));
			}
				
			if($t_target_charset != 'UTF-8')
			{
				if(function_exists('iconv'))
				{
					$p_string = iconv("UTF-8", $t_target_charset . "//TRANSLIT", $p_string);
				}
				else
				{
					$p_string = utf8_decode($p_string);
				}
			}
		}
		
		return $p_string;		
	}
	
	
	function add_xml_child(SimpleXMLElement $p_xml_parent, SimpleXMLElement $p_xml_child, $p_include_xml_declaration = true)
	{
		$t_xml = (string) $p_xml_child->asXML();

		if($p_include_xml_declaration == false)
		{
			$t_xml_line_array = explode("\n", $t_xml);
			array_shift($t_xml_line_array);
			$t_xml = implode("\n", $t_xml_line_array);
		}
		
		$t_dom_parent = dom_import_simplexml($p_xml_parent);
		$t_dom_child = $t_dom_parent->ownerDocument->createDocumentFragment();
		$t_dom_child->appendXML($t_xml);

		$t_success = (boolean) $t_dom_parent->appendChild($t_dom_child);
		return $t_success;
	}
	
	
	function delete_xml_children_by_tag_name(SimpleXMLElement $p_xml_parent, $p_tag_name)
	{
		$t_dom_parent = dom_import_simplexml($p_xml_parent);
		for($i = 0; $i < count($t_dom_parent->getElementsByTagName($p_tag_name)); $i++)
		{
			$t_actual = $t_dom_parent->getElementsByTagName($p_tag_name)->item($i);
			if(!empty($t_actual))
			{
				$t_dom_parent->removeChild($t_actual);
				$i--;
			}
		}
	}
	
	public function wrap_value($p_value, $p_prefix = '<![CDATA[', $p_suffix = ']]>')
	{
		$t_wrapped_value = $p_value;
		
		if($p_value !== '')
		{
			$t_wrapped_value = $p_prefix . $p_value . $p_suffix;
		}
		
		return $t_wrapped_value;
	}
}
