<?php
/* --------------------------------------------------------------
  GxmlProperties.inc.php 2014-01-31 gm
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2014 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------
 */

class GxmlProperties extends GxmlMaster
{

	public function __construct() 
	{
		parent::__construct();
		$this->set_plural_name('properties');
		$this->set_singular_name('property');
		$this->build_mapper_array();
		$this->build_table_mapper_array();
		$this->load_languages();
	}
	
	protected function build_mapper_array()
	{
		$this->v_mapper_array = array();
		
		$this->v_mapper_array['property_id'] = 'properties_id';
		$this->v_mapper_array['sort_order'] = 'sort_order';
		
		$this->v_mapper_array['name'] = 'properties_name';
		$this->v_mapper_array['admin_name'] = 'properties_admin_name';
	}
	
	protected function build_table_mapper_array()
	{
		$this->v_table_mapper_array = array();
		
		$this->v_table_mapper_array['property_id'] = 'properties';
		$this->v_table_mapper_array['sort_order'] = 'properties';
		
		$this->v_table_mapper_array['name'] = 'properties_description';
		$this->v_table_mapper_array['admin_name'] = 'properties_description';
                
		$this->v_language_dependent_mapper_array = array('properties_description');
	}
	
	function download_properties($p_coo_request_xml)
	{
		$t_parameters_array = $this->build_sql_strings($p_coo_request_xml->parameters, array('limit', 'where'));
		
		$this->v_coo_response_xml->addChild('request');
		$this->v_coo_response_xml->request->addChild('success', 1);
		
		$this->add_properties_block($t_parameters_array);
				
		return $this->v_coo_response_xml;
	}
	
	function upload_properties($p_coo_request_xml)
	{
		$t_response_array = array();
		$t_property_values_output_xml = array();
		
		foreach($p_coo_request_xml->parameters->properties->children() as $t_property_xml)
		{
			$this->upload($t_property_xml, $t_response_array);
			if (isset($t_property_xml->property_values))
			{
				$coo_gxml_property_values = MainFactory::create_object( 'GxmlPropertyValues' );
				$t_property_values_output_xml[] = $coo_gxml_property_values->upload_property_values($t_property_xml, (int) $t_property_xml->property_id);
			}
		}
		$t_output_xml = $this->build_response_xml($t_response_array);
		foreach ($t_property_values_output_xml as $t_property_value_output_xml)
		{
			$t_property_values_children = $t_property_value_output_xml->property_values->children();
			$t_property_values_xml = $t_output_xml->addChild('property_values');
			foreach ($t_property_values_children as $t_property_value_child)
			{
				$this->add_xml_child($t_property_values_xml, $t_property_value_child);
			}
		}
		
		return $t_output_xml;
	}
	
	function add_properties_block($p_parameters_array = array())
	{
		$this->v_coo_response_xml->addChild('properties');
		
		$t_where_part = $this->build_where_part($p_parameters_array);
		$t_limitation = '';
		$t_property_ids_array = true;
		
		if(count($p_parameters_array) > 0)
		{
			$t_property_ids_array = array();
			
			$t_limit_part = ' GROUP BY properties.properties_id' . $this->build_limit_part($p_parameters_array);
			
			$t_sql = "SELECT properties.properties_id
							FROM properties 
							LEFT JOIN properties_description USING (properties_id)
							" . $t_where_part . "
							" . $t_limit_part;
			$t_result = xtc_db_query($t_sql);
			while($t_result_array = xtc_db_fetch_array($t_result))
			{
				$t_property_ids_array[] = $t_result_array['properties_id'];
			}
			
			$t_limit = 0;
			$t_offset = 0;
			if(isset($p_parameters_array['offset']) && !empty($p_parameters_array['offset']))
			{
				$t_offset = (int)$p_parameters_array['offset'];
			}

			if(isset($p_parameters_array['limit']) && !empty($p_parameters_array['limit']))
			{
				$t_limit = (int)$p_parameters_array['limit'];
				
				if(!empty($t_property_ids_array))
				{					
					if($t_where_part == '' )
					{
						$t_limitation = ' WHERE properties.properties_id IN (' . implode(',', $t_property_ids_array) . ') ';
					}
					else
					{
						$t_limitation .= ' AND properties.properties_id IN (' . implode(',', $t_property_ids_array) . ') ';
					}
				}
			}			
		}
		
		if(empty($t_property_ids_array) == false)
		{
			$t_properties_sql = "SELECT * 
										FROM properties 
										LEFT JOIN properties_description USING (properties_id)
										" . $t_where_part . "
											" . $t_limitation;
			$t_properties_result = xtc_db_query($t_properties_sql);

			$t_last_property_id = 0;
			while ($t_property_row = xtc_db_fetch_array($t_properties_result))
			{
				// check if new property
				if($t_last_property_id != $t_property_row['properties_id'])
				{
					$coo_property_child = $this->v_coo_response_xml->properties->addChild('property');
				}

				// add property data
				$this->add($t_property_row, $coo_property_child);

				// check if new property
				if($t_last_property_id != $t_property_row['properties_id'])
				{
					$coo_property_values_child = $coo_property_child->addChild('property_values');
					// add values
					$coo_gxml_property_values = MainFactory::create_object( 'GxmlPropertyValues' );
					$coo_gxml_property_values->add_property_value_block( array( 'property_child' => &$coo_property_values_child, 'property_id' => $t_property_row['properties_id'] ) );
				}

				$t_last_property_id = $t_property_row['properties_id'];
			}
		}
	}
	
	public function add_node($p_data_array, $p_table_column, $p_xml_name, $p_coo_xml)
	{
		switch($p_xml_name)
		{
			case 'name':
			case 'admin_name':
				if($this->node_exists($p_coo_xml, $p_xml_name, array('language_id' => $p_data_array['language_id'])) === false)
				{
					$p_coo_xml->addChild($p_xml_name);
					$p_coo_xml->{$p_xml_name}[count($p_coo_xml->{$p_xml_name}) - 1] = $this->wrap_value($p_data_array[$p_table_column]);
					$coo_child = $p_coo_xml->{$p_xml_name}[count($p_coo_xml->{$p_xml_name}) - 1];
					$coo_child['language_id'] = $p_data_array['language_id'];
					$coo_child['language_iso'] = $this->v_language_array[ $p_data_array['language_id'] ]['iso'];
				}
				break;
				
			default:
				if($this->node_exists($p_coo_xml, $p_xml_name) === false)
				{
					$p_coo_xml->{$p_xml_name} = $p_data_array[$p_table_column];
				}
				break;
		}
	}
	
	protected function delete_object(SimpleXMLElement $p_coo_xml)
	{
		if(!isset($p_coo_xml->property_id) || empty($p_coo_xml->property_id))
		{
			return false;
		}
		
		$t_property_id	= (int)$p_coo_xml->property_id;
		$t_success		= true;
		
		$t_sql = '
				DELETE
					pv.*, pvd.*, ppcv.*, ppc.*, p.*, pd.*, ppas.*, ppi.*
				FROM
					properties_values pv,
					properties_values_description pvd,
					products_properties_combis_values ppcv,
					products_properties_combis ppc,
					properties p,
					properties_description pd,
					products_properties_admin_select ppas,
					products_properties_index ppi
				WHERE
					pv.properties_id = ' . $t_property_id . ' AND
					pv.properties_values_id = pvd.properties_values_id AND
					ppcv.properties_values_id = pv.properties_values_id AND
					pv.properties_values_id = ppas.properties_values_id AND
					ppc.products_properties_combis_id = ppcv.products_properties_combis_id AND
					p.properties_id = pv.properties_id AND
					pv.properties_id = pd.properties_id AND
					ppi.products_properties_combis_id = ppc.products_properties_combis_id
				';
		$t_success &= $this->perform_db_action($t_sql);
		
		return $t_success;
	}
}
