<?php
/* --------------------------------------------------------------
  GxmlPropertyValues.inc.php 2014-01-31 gm
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2014 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------
 */

class GxmlPropertyValues extends GxmlMaster
{

	public function __construct() 
	{
		parent::__construct();
		$this->set_plural_name('property_values');
		$this->set_singular_name('property_value');
		$this->build_mapper_array();
		$this->build_table_mapper_array();
		$this->load_languages();
	}
	
	protected function build_mapper_array()
	{
		$this->v_mapper_array = array();
		$this->v_mapper_array['property_value_id'] = 'properties_values_id';
		$this->v_mapper_array['property_id'] = 'properties_id';
		$this->v_mapper_array['sort_order'] = 'sort_order';
		$this->v_mapper_array['model_fragment'] = 'value_model';
		$this->v_mapper_array['price'] = 'value_price';
		$this->v_mapper_array['image'] = 'values_image';
		$this->v_mapper_array['name'] = 'values_name';
	}
	
	protected function build_table_mapper_array()
	{
		$this->v_table_mapper_array = array();
		
		$this->v_table_mapper_array['property_value_id'] = 'properties_values';
		$this->v_table_mapper_array['property_id'] = 'properties_values';
		$this->v_table_mapper_array['sort_order'] = 'properties_values';
		$this->v_table_mapper_array['model_fragment'] = 'properties_values';
		$this->v_table_mapper_array['price'] = 'properties_values';
		$this->v_table_mapper_array['image'] = 'properties_values_description';
		$this->v_table_mapper_array['name'] = 'properties_values_description';
                
		$this->v_language_dependent_mapper_array = array('properties_values_description');
	}
	
	public function add_property_value_block($p_parameters_array = array())
	{
		$t_property_child = $p_parameters_array['property_child'];
		$t_property_id = $p_parameters_array['property_id'];
				
		$t_properties_values_sql = 'SELECT * FROM properties_values LEFT JOIN properties_values_description USING (properties_values_id)';
		$t_properties_values_result = xtc_db_query($t_properties_values_sql);
			
		$t_last_property_value_id = 0;
		while($t_properties_values_row = xtc_db_fetch_array($t_properties_values_result))
		{
			if( $t_property_id == $t_properties_values_row['properties_id'] )
			{
				if($t_last_property_value_id != $t_properties_values_row['properties_values_id'])
				{
					$coo_property_value_child = $t_property_child->addChild('property_value');
				}

				$this->add($t_properties_values_row, $coo_property_value_child);
				$t_last_property_value_id = $t_properties_values_row['properties_values_id'];
			}
		}
	}
	
	function upload_property_values($p_coo_request_xml, $p_property_id)
	{
		$t_response_array = array();
		$count_property_values_child = 0;
		
		foreach($p_coo_request_xml->property_values->children() as $t_property_value_xml)
		{
			$t_property_value_xml->addChild('property_id', $p_property_id);
			$count_property_values_child++;
			$this->upload($t_property_value_xml, $t_response_array);
		}
		$coo_response_xml = $this->build_response_xml($t_response_array);
		
		return $coo_response_xml;
	}
	
	public function add_node($p_data_array, $p_table_column, $p_xml_name, $p_coo_xml)
	{
		switch($p_xml_name)
		{
			case 'image':
			case 'name':
				if($this->node_exists($p_coo_xml, $p_xml_name, array('language_id' => $p_data_array['language_id'])) === false)
				{
					$p_coo_xml->addChild($p_xml_name);
					$p_coo_xml->{$p_xml_name}[count($p_coo_xml->{$p_xml_name}) - 1] = $this->wrap_value($p_data_array[$p_table_column]);
					$coo_child = $p_coo_xml->{$p_xml_name}[count($p_coo_xml->{$p_xml_name}) - 1];
					$coo_child['language_id'] = $p_data_array['language_id'];
					$coo_child['language_iso'] = $this->v_language_array[ $p_data_array['language_id'] ]['iso'];
				}
				break;
			
			case 'model_fragment':
				if($this->node_exists($p_coo_xml, $p_xml_name) === false)
				{
					$p_coo_xml->{$p_xml_name} = $this->wrap_value($p_data_array[$p_table_column]);
				}
				break;
				
			default:
				if($this->node_exists($p_coo_xml, $p_xml_name) === false)
				{
					$p_coo_xml->{$p_xml_name} = $p_data_array[$p_table_column];
				}
				break;
		}
	}
	
	protected function delete_object(SimpleXMLElement $p_coo_xml)
	{
		if(!isset($p_coo_xml->property_value_id) || empty($p_coo_xml->property_value_id))
		{
			return false;
		}
		
		$t_properties_values_id	= (int)$p_coo_xml->property_value_id;
		$t_success				= true;
		
		$t_sql = '
				DELETE
					pv.*, pvd.*, ppcv.*, ppc.*, ppas.*, ppi.*
				FROM
					properties_values pv,
					properties_values_description pvd,
					products_properties_combis_values ppcv,
					products_properties_combis ppc,
					properties p,
					properties_description pd,
					products_properties_admin_select ppas,
					products_properties_index ppi
				WHERE
					pv.properties_values_id = ' . $t_properties_values_id . ' AND
					pv.properties_values_id = pvd.properties_values_id AND
					pvd.properties_values_id = ppcv.properties_values_id AND
					ppcv.products_properties_combis_id = ppc.products_properties_combis_id AND
					pv.properties_values_id = ppas.properties_values_id AND
					ppi.products_properties_combis_id = ppcv.products_properties_combis_id
				';
		
		$t_success &= $this->perform_db_action($t_sql);
		
		return $t_success;
	}
}