<?php
/* --------------------------------------------------------------
   XMLConnectAjaxHandler.inc.php 2014-01-31 gm
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2014 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

function custom_error_handler($errno, $errstr, $errfile, $errline, array $errcontext)
{
	if (error_reporting() === 0)
	{
		return false;
	}

	throw new ErrorException($errstr, 0, $errno, $errfile, $errline);
}

class XMLConnectAjaxHandler extends AjaxHandler
{
	private $v_coo_gxml_helper;
	
	public function __construct() 
	{
		parent::__construct();
		$this->v_coo_gxml_helper = MainFactory::create_object('GxmlHelper');
	}	
	
	function get_permission_status($p_customers_id=NULL)
	{
		return true;
	}
	
	function proceed()
	{
		set_error_handler('custom_error_handler');
		
		$t_request_xml_content = $_POST['gambio_api_xml'];
		$t_request_xml_content = gm_prepare_string($t_request_xml_content, true);
		
		// Debug-Logging
		file_put_contents(DIR_FS_CATALOG . 'logfiles/xml.log', str_repeat('-', 100) . "\nDatum: " . date('Y-m-d H:i:s') . "\nIP: " . $_SERVER['REMOTE_ADDR'] . "\n\nGET-" . print_r($_GET, true) . "\nPOST-" . print_r($_POST, true) . "\ngambio_api_xml:\n" . $t_request_xml_content . "\n", FILE_APPEND);
		
		try
		{
			$coo_request_xml = simplexml_load_string($t_request_xml_content);

			$t_function = (string)$coo_request_xml->general->function;
			$coo_response_xml = false;

			$t_valid_request = true;

			if(!in_array($t_function, array('login', 'logout')))
			{
				$coo_auth = MainFactory::create_object('GxmlAuth');
				$t_valid_request = $coo_auth->validate_session_key($coo_request_xml->general->session_key);			
			}

			if($t_valid_request)
			{
				switch($t_function)
				{
					case 'login':
						$coo_gxml = MainFactory::create_object('GxmlAuth');
						$coo_response_xml = $coo_gxml->login($coo_request_xml);
						break;

					case 'logout':
						$coo_gxml = MainFactory::create_object('GxmlAuth');
						$coo_response_xml = $coo_gxml->logout($coo_request_xml);
						break;

					case 'download_categories':
						$coo_gxml = MainFactory::create_object('GxmlCategories');
						$coo_response_xml = $coo_gxml->download_categories($coo_request_xml);
						break;

					case 'download_products':
						$coo_gxml = MainFactory::create_object('GxmlProducts');
						$coo_response_xml = $coo_gxml->download_products($coo_request_xml);
						break;

					case 'download_orders':
						$coo_gxml = MainFactory::create_object('GxmlOrders');
						$coo_response_xml = $coo_gxml->download_orders($coo_request_xml);
						break;

					case 'download_tax_classes':
						$coo_gxml = MainFactory::create_object('GxmlTaxClasses');
						$coo_response_xml = $coo_gxml->download_tax_classes();
						break;

					case 'download_languages':
						$coo_gxml = MainFactory::create_object('GxmlLanguages');
						$coo_response_xml = $coo_gxml->download_languages();
						break;

					case 'download_order_status':
						$coo_gxml = MainFactory::create_object('GxmlOrderStatus');
						$coo_response_xml = $coo_gxml->download_order_status();
						break;

					case 'upload_categories':
						$coo_gxml = MainFactory::create_object('GxmlCategories');
						$coo_response_xml = $coo_gxml->upload_categories($coo_request_xml);
						break;

					case 'upload_products':
						$coo_gxml = MainFactory::create_object('GxmlProducts');
						$coo_response_xml = $coo_gxml->upload_products($coo_request_xml);
						break;

					case 'upload_order_status':
						$coo_gxml = MainFactory::create_object('GxmlOrders');
						$coo_response_xml = $coo_gxml->upload_order_status($coo_request_xml);
						break;

					case 'download_properties':
						$coo_gxml = MainFactory::create_object('GxmlProperties');
						$coo_response_xml = $coo_gxml->download_properties($coo_request_xml);
						break;

					case 'upload_properties':
						$coo_gxml = MainFactory::create_object('GxmlProperties');
						$coo_response_xml = $coo_gxml->upload_properties($coo_request_xml);
						break;
				}
			}
			else
			{
				$coo_response_xml = $coo_auth->build_invalid_session_response();
			}
		}
		catch (ErrorException $p_exception)
		{
			$coo_response_xml = simplexml_load_string($this->v_coo_gxml_helper->get_blank_xml());
			$coo_response_xml->addChild('request');
			$coo_response_xml->request->addChild('success', 0);
			
			$t_error_message = $p_exception;
			if(strpos($p_exception, 'simplexml_load_string') !== false && strpos($p_exception, 'Opening and ending tag mismatch') !== false)
			{
				$t_error_message = 'Request invalid. Opening and ending tag mismatch.';
			}
			else if(strpos($p_exception, 'simplexml_load_string') !== false && strpos($p_exception, 'xmlParseEntityRef') !== false)
			{
				$t_error_message = 'Request invalid. Use of illegal character.';
			}
			else if(strpos($p_exception, 'simplexml_load_string') !== false && strpos($p_exception, 'EntityRef: expecting \';\'') !== false)
			{
				$t_error_message = 'Request invalid. HTML entity fragment without delimiter \';\' found (e.g. &amp).';
			}
			
			$t_error_message = '<![CDATA[' . $t_error_message . ']]>';
			
			$coo_response_xml->request->addChild('errormessage', $t_error_message);
		}
		restore_error_handler();
		
		if($coo_response_xml === false) 
		{
			file_put_contents(DIR_FS_CATALOG . 'logfiles/xml_error.log', str_repeat('-', 100) . "\nDatum: " . date('Y-m-d H:i:s') . "\nIP: " . $_SERVER['REMOTE_ADDR'] . "\n\nGET-" . print_r($_GET, true) . "\nPOST-" . print_r($_POST, true) . "\ngambio_api_xml:\n" . $t_request_xml_content . "\n", FILE_APPEND);
			
			$coo_response_xml = simplexml_load_string($this->v_coo_gxml_helper->get_blank_xml());
			$coo_response_xml->addChild('request');
			$coo_response_xml->request->addChild('success', 0);
			
			$t_error_message = 'An unknown error occurred for function \'' . $t_function . '\': ' . $t_request_xml_content;
			
			$t_error_message = '<![CDATA[' . $t_error_message . ']]>';
			
			$coo_response_xml->request->addChild('errormessage', $t_error_message);
		}
		
		$this->add_header('Content-Type: text/xml;');
		$this->v_output_buffer = $coo_response_xml->asXML();
		$this->v_output_buffer = utf8_encode($this->v_output_buffer);
		
		return true;
	}
}
