<?php
/* --------------------------------------------------------------
  LanguageTextManager.inc.php 2014-07-25 gambio
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2014 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------
 */

/*
  Example:
  $coo_text_manager = MainFactory::create_object('LanguageTextManager', array('index', $_SESSION['languages_id']), true);
  echo 'test: '. $coo_text_manager->get_text('wishlist');
 */

class LanguageTextManager
{
	var $v_default_section = '';
	var $v_default_language_id = 0;
	var $v_use_fallback;
	var $v_section_content_array = array();
	var $v_languages = array();

	static function get_instance($p_default_section = false, $p_default_language_id = false, $p_use_fallback = true)
	{
		static $s_instance;

		if($s_instance === NULL)
		{
			$s_instance = MainFactory::create_object('LanguageTextManager', array($p_default_section, $p_default_language_id, $p_use_fallback));
		}
		else
		{
			if($s_instance->v_default_section != $p_default_section || $s_instance->v_default_language_id != $p_default_language_id)
			{
				# re-init with new parameters
				$s_instance->init_section($p_default_section, $p_default_language_id);
			}
		}
		return $s_instance;
	}

	function LanguageTextManager($p_default_section = false, $p_default_language_id = false, $p_use_fallback = true)
	{
		$this->load_languages();

		if((int)$p_default_language_id > 0)
		{
			$this->v_default_language_id = (int)$p_default_language_id;
		}
		
		$this->v_use_fallback = (bool)$p_use_fallback;

		if($p_default_section != false)
		{
			$this->v_default_section = $p_default_section;
			$this->v_default_language_id = (int)$p_default_language_id;

			$t_cache_key = 'LanguageTextManager' . $p_default_language_id . $p_default_section . (int)$this->v_use_fallback;
			$coo_cache = & DataCache::get_instance();
			if($coo_cache->key_exists($t_cache_key, true))
			{
				#use cached data
				$this->v_section_content_array = $coo_cache->get_data($t_cache_key);
			}
			else
			{
				#build new content
				$this->init_section($this->v_default_section, $this->v_default_language_id);
				$coo_cache->set_data($t_cache_key, $this->v_section_content_array, true);
			}
		}
	}

	function load_languages()
	{
		$t_sql = 'SELECT * FROM ' . TABLE_LANGUAGES;
		$t_result = xtc_db_query($t_sql);
		while($t_row = xtc_db_fetch_array($t_result))
		{
			$this->v_languages[$t_row['code']] = array();
			$this->v_languages[$t_row['code']]['language_id'] = $t_row['languages_id'];
			$this->v_languages[$t_row['code']]['directory'] = $t_row['directory'];
		}
	}

	function init_from_files($p_section, $p_language_id)
	{
		$c_language_id = (int)$p_language_id;
		$t_directory = '';
		$t_default_directory = $this->v_languages[DEFAULT_LANGUAGE]['directory'];

		if($c_language_id == $_SESSION['languages_id'])
		{
			$t_directory = $_SESSION['language'];
		}
		else
		{
			$t_sql = 'SELECT directory FROM ' . TABLE_LANGUAGES . ' WHERE languages_id = ' . $c_language_id;
			$t_query = xtc_db_query($t_sql);
			if(xtc_db_num_rows($t_query) == 1)
			{
				$t_result = xtc_db_fetch_array($t_query);
				$t_directory = $t_result['directory'];
			}
		}
		
		if($this->is_lang_file($p_section))
		{
			$t_section = $this->convert_lang_file_path_to_section($p_section);
		}
		else
		{
			$t_section = $p_section;
		}

		$t_sections_path = DIR_FS_CATALOG . 'lang/' . $t_directory . '/sections/';
		$t_default_sections_path = DIR_FS_CATALOG . 'lang/' . $t_default_directory . '/sections/';
		$t_lang_file_suffix = 'lang.inc.php';

		$t_lang_files = glob($t_sections_path . $t_section . '.*' . $t_lang_file_suffix);
		$t_is_fallback = false;

		if(is_array($t_lang_files) === false && $this->v_use_fallback)
		{
			$t_lang_files = glob($t_default_sections_path . $t_section . '.*' . $t_lang_file_suffix);
			$t_is_fallback = true;
		}
		
		if(is_array($t_lang_files))
		{
			# target for included lang files     
			foreach($t_lang_files as $t_lang_file)
			{
				$t_language_text_section_content_array = array();

				$t_default_section_path = str_replace(preg_replace('/\//', '\\/', $t_sections_path), $t_default_sections_path, $t_lang_file);
				if(file_exists($t_default_section_path))
				{
					include($t_default_section_path);
				}
				$t_fallback_array = $t_language_text_section_content_array;

				$t_language_text_section_content_array = array();
				if(file_exists($t_lang_file))
				{
					include($t_lang_file);
				}
				
				if($t_is_fallback)
				{
					$t_section_fallback_diff = array_diff_key($t_fallback_array, $t_language_text_section_content_array);
					$t_language_text_section_content_array = array_diff_key(array_merge($t_language_text_section_content_array, $t_section_fallback_diff), $this->v_section_content_array[$p_section]);
				}
				else
				{
					$t_language_text_section_content_array = array_merge($t_language_text_section_content_array, array_diff_key($t_fallback_array, $t_language_text_section_content_array));
				}
				
				if(sizeof($t_language_text_section_content_array) > 0)
				{
					$this->add_section($p_section, $t_language_text_section_content_array);
				}
			}
		}
	}

	function init_from_database($p_section, $p_language_id)
	{
		$t_section_array = array();
		$t_language_code = DEFAULT_LANGUAGE;

		foreach($this->v_languages as $t_key => $t_lang_data)
		{
			if($t_lang_data['language_id'] == $p_language_id)
			{
				$t_language_code = $t_key;
				break;
			}
		}

		# get section content from database
		$t_sql = '
			SELECT
				lf.language_id,
				lfc.phrase_name,
				lfc.phrase_value
			FROM
				language_sections AS lf,
				language_section_phrases AS lfc
			WHERE
				lf.language_section_id = lfc.language_section_id AND
				(lf.section_name = "' . xtc_db_input($p_section) . '"
		';
		
		if($this->v_use_fallback)
		{
			if($this->is_lang_file($p_section))
			{
				$t_new_file_path = $this->convert_lang_file_path_by_language_id($p_section, $this->v_languages[DEFAULT_LANGUAGE]['language_id']);
				if($t_new_file_path !== false)
				{
					$t_sql .= ' OR lf.section_name = "' . xtc_db_input($t_new_file_path) . '") AND ';
				}
			}
			else
			{
				$t_sql .= ') AND ';
			}
			$t_sql .= 'lf.language_id IN (' . (int)$p_language_id . ', ' . $this->v_languages[DEFAULT_LANGUAGE]['language_id'] . ')';
		}
		else
		{
			$t_sql .= ') AND ';
			$t_sql .= 'lf.language_id  = "' . addslashes($p_language_id) . '"';
		}

		$t_result = xtc_db_query($t_sql);
		while(($t_row = xtc_db_fetch_array($t_result)))
		{
			# collect result in section array
			$t_var_name = $t_row['phrase_name'];
			$t_var_value = $t_row['phrase_value'];
			if(!empty($t_section_array[$t_var_name]) && $t_row['language_id'] == $this->v_languages[DEFAULT_LANGUAGE]['language_id'])
			{
				continue;
			}
			$t_section_array[$t_var_name] = $t_var_value;
		}
		# add section content to object cache
		if(sizeof($t_section_array) > 0)
		{
			$this->add_section($p_section, $t_section_array);
		}
	}

	function init_from_lang_file($p_file_path)
	{
		$c_language_id = $this->get_lang_file_language_id($p_file_path);
		if($c_language_id === false)
		{
			$c_language_id = $this->v_default_language_id;
		}
		$this->init_section($p_file_path, $c_language_id);
	}
	
	protected function include_lang_file($p_file_path)
	{
		$t_path = DIR_FS_CATALOG . $p_file_path;
		if(file_exists($t_path))
		{
			include_once($t_path);
		}
	}

	function init_section($p_section, $p_language_id = false)
	{
		$c_section = addslashes($p_section);

		$coo_stop_watch = LogControl::get_instance()->get_stop_watch();
		$coo_stop_watch->start('init_section');

		$this->reset_section($c_section);
		
		$c_language_id = (int)$p_language_id;
		
		if($this->is_lang_file($c_section))
		{
			$this->init_from_files($c_section, $c_language_id);
			$this->init_constants($c_section);
			$this->include_lang_file($c_section);
			$this->init_from_database($c_section, $c_language_id);
			$this->init_constants($c_section);
		}
		else
		{
			$this->init_from_database($c_section, $c_language_id);
			$this->init_from_files($c_section, $c_language_id);
		}
		
		$coo_stop_watch->stop('init_section');
	}

	function reset_section($p_section)
	{
		$this->v_section_content_array[$p_section] = array();
	}

	function add_section($p_section, $p_section_array)
	{
		$this->v_section_content_array[$p_section] = array_merge($this->v_section_content_array[$p_section], $p_section_array);
	}

	function get_text($p_var_name, $p_section = false, $p_language_id = false)
	{
		if($p_section === false)
		{
			$t_section = $this->v_default_section;
		}
		else
		{
			$t_section = $p_section;
		}
		if($p_language_id === false)
		{
			$t_language_id = $this->v_default_language_id;
		}
		else
		{
			$t_language_id = $p_language_id;
		}

		# section content already available?
		if(isset($this->v_section_content_array[$t_section]) == false)
		{
			# do init if not
			$this->init_section($t_section, $t_language_id);
		}

		# get var value and return
		$t_var_value = $p_var_name;
		if(isset($this->v_section_content_array[$t_section][$p_var_name]))
		{
			$t_var_value = $this->v_section_content_array[$t_section][$p_var_name];
		}
		return $t_var_value;
	}

	function get_section_array($p_section = false, $p_language_id = false)
	{
		if($p_section === false)
		{
			$t_section = $this->v_default_section;
		}
		else
		{
			$t_section = $p_section;
		}
		if($p_language_id === false)
		{
			$t_language_id = $this->v_default_language_id;
		}
		else
		{
			$t_language_id = $p_language_id;
		}

		# section content already available?
		if(isset($this->v_section_content_array[$t_section]) == false)
		{
			# do init if not
			$this->init_section($t_section, $t_language_id);
		}

		$t_section_array = $this->v_section_content_array[$t_section];
		return $t_section_array;
	}
	
	protected function is_lang_file($p_lang_file)
	{
		return $this->get_lang_file_language_data($p_lang_file) !== false;
	}
	
	protected function get_lang_file_language_data($p_lang_file)
	{
		foreach($this->v_languages as $t_lang_data)
		{
			if(strpos($p_lang_file, 'lang/' . $t_lang_data['directory'] . '/') !== false)
			{
				return $t_lang_data;
			}
		}
		return false;
	}
	
	protected function get_lang_file_language_data_by_language_id($p_language_id)
	{
		foreach($this->v_languages as $t_lang_data)
		{
			if($t_lang_data['language_id'] == (int)$p_language_id)
			{
				return $t_lang_data;
			}
		}
		return false;
	}
	
	protected function get_lang_file_language_id($p_lang_file)
	{
		$t_language_data = $this->get_lang_file_language_data($p_lang_file);
		if($t_language_data !== false && is_array($t_language_data) && isset($t_language_data['language_id']))
		{
			return $t_language_data['language_id'];
		}
		return false;
	}
	
	protected function init_constants($p_section)
	{
		foreach($this->v_section_content_array[$p_section] as $t_key => $_value)
		{
			if(defined($t_key) == false)
			{
				define($t_key, $_value);
			}
		}
	}
	
	protected function convert_lang_file_path_by_language_id($p_file_path, $p_language_id)
	{
		$t_old_lang_file_data = $this->get_lang_file_language_data($p_file_path);
		
		if($t_old_lang_file_data === false)
		{
			return false;
		}
		
		$t_new_lang_file_data = $this->get_lang_file_language_data_by_language_id($p_language_id);
		
		if($t_new_lang_file_data === false)
		{
			return false;
		}
		
		$t_old_path = 'lang/' . $t_old_lang_file_data['directory'] . '/';
		$t_new_path = 'lang/' . $t_new_lang_file_data['directory'] . '/';
		$t_file_path = str_replace($t_old_path, $t_new_path, $p_file_path);
		
		return $t_file_path;
	}
	
	protected function convert_lang_file_path_to_section($p_file_path)
	{
		$t_section = str_replace('/', '__', $p_file_path);
		$t_section = str_replace('.', '___', $t_section);
		return $t_section;
	}
}