<?php
/* --------------------------------------------------------------
   GoogleRichSnippetContentView.inc.php 2014-09-08 gm
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2014 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

class GoogleRichSnippetContentView extends ContentView
{
	protected $active;
	protected $fsk18;
	protected $price_status;
	protected $quantity;
	protected $date_available;
	protected $price;
	protected $currency;
	protected $products_name;
	protected $breadcrumb_array;
	protected $breadcrumb_separator;
	protected $review_date_created;
	protected $rating;

	
	public function __construct()
	{
		parent::__construct();
		
		$this->active = true;
	}

	protected function set_validation_rules()
	{
		$this->validation_rules_array['active']					= array('type'		=> 'bool',
																		'strict'	=> true);
		$this->validation_rules_array['fsk18']					= array('type'		=> 'bool',
																		'strict'	=> true);
		$this->validation_rules_array['breadcrumb_array']		= array('type'		=> 'array');
		$this->validation_rules_array['price_status']			= array('type'		=> 'int');
		$this->validation_rules_array['rating']					= array('type'		=> 'int');
		$this->validation_rules_array['price']					= array('type'		=> 'double');
		$this->validation_rules_array['quantity']				= array('type'		=> 'double');
		$this->validation_rules_array['breadcrumb_separator']	= array('type'		=> 'string');
		$this->validation_rules_array['currency']				= array('type'		=> 'string');
		$this->validation_rules_array['date_available']			= array('type'		=> 'string');
		$this->validation_rules_array['review_date_created']	= array('type'		=> 'string');
		$this->validation_rules_array['products_name']			= array('type'		=> 'string');
	}
	

	protected function is_rich_snippet_active()
	{
		$t_uninitialized_array = $this->get_uninitialized_variables(array('fsk18', 'quantity', 'price_status', 'active'));
		
		if(empty($t_uninitialized_array))
		{
			if($this->fsk18 == false 
					&& (STOCK_ALLOW_CHECKOUT == 'true' || ($this->quantity > 0) && STOCK_ALLOW_CHECKOUT == 'false') 
					&& $this->price_status == '0'
					&& $this->active == true)
			{
				return true;
			}
		}
		else
		{
			trigger_error("Variable(s) " . implode(', ', $t_uninitialized_array) . " do(es) not exist in class " . get_class($this) . " or is/are null", E_USER_ERROR);
		}
		
		return false;
	}

	
	public function get_product_snippet()
	{
		$t_rich_snippet_array = array();
		$t_rich_snippet_array['product_itemscope'] = '';
		$t_rich_snippet_array['product_itemprop_image'] = '';
		$t_rich_snippet_array['product_itemprop_name_start'] = '';
		$t_rich_snippet_array['product_itemprop_model_start'] = '';
		$t_rich_snippet_array['product_itemprop_model_end'] = '';
		$t_rich_snippet_array['product_itemprop_model_end'] = '';
		$t_rich_snippet_array['product_itemprop_description_start'] = '';
		$t_rich_snippet_array['product_itemprop_description_end'] = '';
		$t_rich_snippet_array['product_itemprop_offers'] = '';

		$t_rich_snippet_array['offer_itemscope'] = '';
		$t_rich_snippet_array['offer_itemprop_price_start'] = '';
		$t_rich_snippet_array['offer_itemprop_price_end'] = '';
		$t_rich_snippet_array['offer_itemprop_price_currency'] = '';

		
		$t_uninitialized_array = $this->get_uninitialized_variables(array('price', 'currency'));
		
		if(empty($t_uninitialized_array))
		{
			if($this->is_rich_snippet_active())
			{
	//			if(empty($this->date_available) == false && $this->date_available > date('Y-m-d H:i:s'))
	//			{
	//				$t_rich_snippet_array['availability'] = 'preorder';
	//			}
	//			elseif($this->quantity > 0)
	//			{
	//				$t_rich_snippet_array['availability'] = 'in_stock';
	//			}
	//			elseif($this->quantity <= 0 )
	//			{
	//				$t_rich_snippet_array['availability'] = 'out_of_stock';
	//			}


				$t_rich_snippet_array['product_itemscope'] = ' itemscope="itemscope" itemtype="http://schema.org/Product"';
				$t_rich_snippet_array['product_itemprop_image'] = ' itemprop="image"';
				$t_rich_snippet_array['product_itemprop_name_start'] = '<span itemprop="name">';
				$t_rich_snippet_array['product_itemprop_name_end'] = '</span>';
				$t_rich_snippet_array['product_itemprop_model_start'] = '<span itemprop="model">';
				$t_rich_snippet_array['product_itemprop_model_end'] = '</span>';
				$t_rich_snippet_array['product_itemprop_description_start'] = '<div itemprop="description">';
				$t_rich_snippet_array['product_itemprop_description_end'] = '</div>';
				$t_rich_snippet_array['product_itemprop_offers'] = ' itemprop="offers"';

				$t_rich_snippet_array['offer_itemscope'] = ' itemscope="itemscope" itemtype="http://schema.org/Offer"';
				$t_rich_snippet_array['offer_itemprop_price_start'] = '<span itemprop="price" content="' .  number_format($this->price, 2, '.', '') . '">';
				$t_rich_snippet_array['offer_itemprop_price_end'] = '</span>';
				$t_rich_snippet_array['offer_itemprop_price_currency'] = '<meta itemprop="priceCurrency" content="' . $this->currency .'" />';
			}
			
		}
		else
		{
			trigger_error("Variable(s) " . implode(', ', $t_uninitialized_array) . " do(es) not exist in class " . get_class($this) . " or is/are null", E_USER_ERROR);
		}
		
		return $t_rich_snippet_array;
	}
	
	
	function get_breadcrumb_snippet()
	{
		$t_output_html = '';
		
		$t_uninitialized_array = $this->get_uninitialized_variables(array('breadcrumb_array', 'breadcrumb_separator', 'active'));
		
		if(empty($t_uninitialized_array))
		{
			$t_items_array = array();
			$t_breadcrumb_array = $this->breadcrumb_array;

			if($this->active == true)
			{
				$t_output_html .= '<div id="breadcrumb_navi" itemscope="itemscope" itemtype="http://schema.org/breadcrumb">';

				foreach($t_breadcrumb_array as $t_value_array)
				{
					if(isset($t_value_array['link']) && xtc_not_null($t_value_array['link']))
					{
						$t_items_array[] = '<a href="' . $t_value_array['link'] . '" class="headerNavigation" itemprop="url">
												<span itemprop="title">' . $t_value_array['title'] . '</span>
											</a>';
					}
					else
					{
						$t_items_array[] = '<span itemprop="title">' . $t_value_array['title'] . '</span>';
					}
				}

				$t_output_html .= implode($this->breadcrumb_separator, $t_items_array);

				$t_output_html .= '</div>';
			}
			else
			{
				$t_output_html .= '<div id="breadcrumb_navi">';

				foreach($t_breadcrumb_array as $t_value_array)
				{
					if(isset($t_value_array['link']) && xtc_not_null($t_value_array['link']))
					{
						$t_items_array[] = '<a href="' . $t_value_array['link'] . '" class="headerNavigation">
												<span>' . $t_value_array['title'] . '</span>
											</a>';
					}
					else
					{
						$t_items_array[] = '<span>' . $t_value_array['title'] . '</span>';
					}
				}

				$t_output_html .= implode($this->breadcrumb_separator, $t_items_array);

				$t_output_html .= '</div>';
			}
		}
		else
		{
			trigger_error("Variable(s) " . implode(', ', $t_uninitialized_array) . " do(es) not exist in class " . get_class($this) . " or is/are null", E_USER_ERROR);
		}
		
		return $t_output_html;
	}
	
	
	public function get_review_snippet()
	{
		$t_rich_snippet_array = array();
		$t_rich_snippet_array['product_itemprop_reviews'] = '';
		$t_rich_snippet_array['review_itemscope'] = '';
		$t_rich_snippet_array['review_itemprop_about'] = '';
		$t_rich_snippet_array['review_itemprop_reviewBody'] = '';
		$t_rich_snippet_array['review_itemprop_author'] = '';
		$t_rich_snippet_array['author_itemscope'] = '';
		$t_rich_snippet_array['author_itemprop_name'] = '';
		$t_rich_snippet_array['review_itemprop_datePublished'] = '';
		$t_rich_snippet_array['review_itemprop_reviewRating'] = '';
		$t_rich_snippet_array['rating_itemscope'] = '';
		$t_rich_snippet_array['rating_itemprop_worstRating'] = '';
		$t_rich_snippet_array['rating_itemprop_ratingValue'] = '';
		$t_rich_snippet_array['rating_itemprop_bestRating'] = '';
		
		
		$t_uninitialized_array = $this->get_uninitialized_variables(array('products_name', 'review_date_created', 'rating'));
		
		if(empty($t_uninitialized_array))
		{
			if($this->is_rich_snippet_active())
			{
				$t_rich_snippet_array['product_itemprop_reviews'] = ' itemprop="reviews"';

				$t_rich_snippet_array['review_itemscope'] = ' itemscope="itemscope" itemtype="http://schema.org/Review"';
				$t_rich_snippet_array['review_itemprop_about'] = ' <meta itemprop="about" content="' . $this->products_name . '">';
				$t_rich_snippet_array['review_itemprop_reviewBody'] = ' itemprop="reviewBody"';
				$t_rich_snippet_array['review_itemprop_author'] = ' itemprop="author"';

				$t_rich_snippet_array['author_itemscope'] = ' itemscope="itemscope" itemtype="http://schema.org/Person"';
				$t_rich_snippet_array['author_itemprop_name'] = ' itemprop="name"';

				$t_rich_snippet_array['review_itemprop_datePublished'] = '<meta itemprop="datePublished" content="' . $this->review_date_created . '">';
				$t_rich_snippet_array['review_itemprop_reviewRating'] = ' itemprop="reviewRating"';

				$t_rich_snippet_array['rating_itemscope'] = ' itemscope="itemscope" itemtype="http://schema.org/Rating"';
				$t_rich_snippet_array['rating_itemprop_worstRating'] = '<meta itemprop="worstRating" content="1">';
				$t_rich_snippet_array['rating_itemprop_ratingValue'] = '<meta itemprop="ratingValue" content="' . $this->rating . '">';
				$t_rich_snippet_array['rating_itemprop_bestRating'] = '<meta itemprop="bestRating" content="5">';
			}
		}
		else
		{
			trigger_error("Variable(s) " . implode(', ', $t_uninitialized_array) . " do(es) not exist in class " . get_class($this) . " or is/are null", E_USER_ERROR);
		}
		
		return $t_rich_snippet_array;
	}
}