<?php
/* --------------------------------------------------------------
   ProductReviewsInfoContentView.inc.php 2014-02-28 gambio
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2014 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------


   based on: 
   (c) 2000-2001 The Exchange Project  (earlier name of osCommerce)
   (c) 2002-2003 osCommerce(product_reviews_info.php,v 1.47 2003/02/13); www.oscommerce.com
   (c) 2003	 nextcommerce (product_reviews_info.php,v 1.12 2003/08/17); www.nextcommerce.org
   (c) 2003 XT-Commerce - community made shopping http://www.xt-commerce.com ($Id: product_reviews_info.php 1238 2005-09-24 10:51:19Z mz $) 

   Released under the GNU General Public License 
   ---------------------------------------------------------------------------------------*/

// include needed functions
require_once (DIR_FS_INC.'xtc_break_string.inc.php');
require_once (DIR_FS_INC.'xtc_date_long.inc.php');

class ProductReviewsInfoContentView extends ContentView
{
	protected $review_data_array;
	protected $coo_seo_boost;
	
	public function __construct()
	{
		parent::__construct();
		$this->set_content_template('/module/product_reviews_info.html');
		$this->set_flat_assigns(true);
	}
	
	protected function set_validation_rules()
	{
		// SET VALIDATION RULES
		$this->validation_rules_array['review_data_array']	= array('type' => 'array');
		$this->validation_rules_array['coo_seo_boost']		= array('type' => 'object',
																	'object_type' => 'GMSEOBoost');
	}
	
	protected function set_deprecated_array()
	{
		$this->deprecated_array[1][] = 'BUTTON_BACK';
		$this->deprecated_array[1][] = 'BUTTON_BUY_NOW';
		$this->deprecated_array[1][] = 'IMAGE';
	}
	
	public function prepare_data()
	{
		$t_uninitialized_array = $this->get_uninitialized_variables(array('review_data_array'));
		if(empty($t_uninitialized_array))
		{
			$this->coo_seo_boost = MainFactory::create_object('GMSEOBoost');
			$this->add_data();
		}
		else
		{
			trigger_error("Variable(s) " . implode(', ', $t_uninitialized_array) . " do(es) not exist in class " . get_class($this) . " or is/are null", E_USER_ERROR);
		}
	}
	
	protected function add_data()
	{
		$this->content_array['PRODUCTS_NAME'] = $this->review_data_array['products_name'];
		$this->content_array['AUTHOR'] = $this->review_data_array['customers_name'];
		$this->content_array['DATE'] = xtc_date_long($this->review_data_array['date_added']);
		
		$reviews_text = xtc_break_string(htmlspecialchars_wrapper($this->review_data_array['reviews_text']), 60, '-<br />');
		$this->content_array['REVIEWS_TEXT'] = nl2br($reviews_text);
		$this->content_array['RATING'] = xtc_image('templates/'.CURRENT_TEMPLATE.'/img/stars_'.$this->review_data_array['reviews_rating'].'.gif', sprintf(TEXT_OF_5_STARS, $this->review_data_array['reviews_rating']));
		
		if($this->coo_seo_boost->boost_products) {
			$gm_seo_product_link = xtc_href_link($this->coo_seo_boost->get_boosted_product_url($this->review_data_array['products_id'], $this->review_data_array['products_name']) );
		} else {
			$gm_seo_product_link = xtc_href_link(FILENAME_PRODUCT_INFO, xtc_product_link($this->review_data_array['products_id'], $this->review_data_array['products_name']));
		}
		$this->content_array['PRODUCTS_LINK'] = $gm_seo_product_link;

		// lets retrieve all $HTTP_GET_VARS keys and values..
		$get_params = xtc_get_all_get_params(array ('reviews_id'));
		$get_params = substr_wrapper($get_params, 0, -1); //remove trailing &
		
		$this->content_array['BUTTON_BACK'] = '<a href="'.xtc_href_link(FILENAME_PRODUCT_REVIEWS, $get_params).'">'.xtc_image_button('button_back.gif', IMAGE_BUTTON_BACK).'</a>';
		$this->content_array['BUTTON_BACK_LINK'] = xtc_href_link(FILENAME_PRODUCT_REVIEWS, $get_params);
		$this->content_array['BUTTON_BUY_NOW'] = '<a href="'.xtc_href_link(FILENAME_DEFAULT, 'action=buy_now&BUYproducts_id='.$this->review_data_array['products_id']).'">'.xtc_image_button('button_in_cart.gif', IMAGE_BUTTON_IN_CART).'</a>';
		$this->content_array['BUTTON_BUY_NOW_URL'] = xtc_href_link(FILENAME_DEFAULT, 'action=buy_now&BUYproducts_id=' . $this->review_data_array['products_id']);
		$this->content_array['IMAGE'] = xtc_image(DIR_WS_THUMBNAIL_IMAGES.$this->review_data_array['products_image'], $this->review_data_array['products_name'], '', '', 'hspace="5" vspace="5"');
		$this->content_array['IMAGE_URL'] = DIR_WS_THUMBNAIL_IMAGES . basename($this->review_data_array['products_image']);
		
		$this->add_rich_snippet_data();
	}
	
	protected function add_rich_snippet_data()
	{
		$coo_rich_snippet_view = MainFactory::create_object('GoogleRichSnippetContentView');
		$coo_rich_snippet_view->set_('fsk18', (boolean)$this->review_data_array['products_fsk18']);
		$coo_rich_snippet_view->set_('price_status', $this->review_data_array['gm_price_status']);
		$coo_rich_snippet_view->set_('quantity', $this->review_data_array['products_quantity']);
		$coo_rich_snippet_view->set_('review_date_created', date('Y-m-d', strtotime($this->review_data_array['date_added'])));
		$coo_rich_snippet_view->set_('rating',  $this->review_data_array['reviews_rating']);
		$coo_rich_snippet_view->set_('products_name',  $this->review_data_array['products_name']);

		$t_rich_snippet_array = $coo_rich_snippet_view->get_review_snippet();
		$this->content_array['rich_snippet_content'] = $t_rich_snippet_array;
	}
}